/***************************************************************************
                           equalizer.cpp
                             -------------------
    begin                : Son Okt 19 2003
    revision             : $Revision: 1.6 $
    last modified        : $Date: 2004/01/11 12:53:27 $ by $Author: juergenk $
    copyright            : (C) 2003-2004 by Juergen  Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kaccel.h>
#include <klocale.h>
#include <kdebug.h>

#include <qlayout.h>
#include <qlabel.h>

#include "equalizer.h"
#include "equalizer.moc"

Equalizer::Equalizer(QWidget *parent, const char *name)
            : KDialogBase(KDialogBase::Plain, i18n("Kaffeine Equalizer Settings"), KDialogBase::Default | KDialogBase::Close, KDialogBase::Close, parent, name, true)
{
  QWidget* page = plainPage();

  QGridLayout* grid = new QGridLayout(page, 2, 10);
  grid->setSpacing(5);

  QLabel* eq30Text = new QLabel("30Hz", page);
  eq30Slider = new QSlider(Qt::Vertical, page);
  eq30Slider->setRange(-100, 100);
  eq30Slider->setSteps(1, 10);
  connect(eq30Slider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq30(int)));
  grid->addWidget(eq30Text, 1, 0);
  grid->addWidget(eq30Slider, 0, 0);

  QLabel* eq60Text = new QLabel("60Hz", page);
  eq60Slider = new QSlider(Qt::Vertical, page);
  eq60Slider->setRange(-100, 100);
  eq60Slider->setSteps(1, 10);
  connect(eq60Slider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq60(int)));
  grid->addWidget(eq60Text, 1, 1);
  grid->addWidget(eq60Slider, 0, 1);

  QLabel* eq125Text = new QLabel("125Hz", page);
  eq125Slider = new QSlider(Qt::Vertical, page);
  eq125Slider->setRange(-100, 100);
  eq125Slider->setSteps(1, 10);
  connect(eq125Slider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq125(int)));
  grid->addWidget(eq125Text, 1, 2);
  grid->addWidget(eq125Slider, 0, 2);

  QLabel* eq250Text = new QLabel("250Hz", page);
  eq250Slider = new QSlider(Qt::Vertical, page);
  eq250Slider->setRange(-100, 100);
  eq250Slider->setSteps(1, 10);
  connect(eq250Slider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq250(int)));
  grid->addWidget(eq250Text, 1, 3);
  grid->addWidget(eq250Slider, 0, 3);

  QLabel* eq500Text = new QLabel("500Hz", page);
  eq500Slider = new QSlider(Qt::Vertical, page);
  eq500Slider->setRange(-100, 100);
  eq500Slider->setSteps(1, 10);
  connect(eq500Slider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq500(int)));
  grid->addWidget(eq500Text, 1, 4);
  grid->addWidget(eq500Slider, 0, 4);

  QLabel* eq1kText = new QLabel("1kHz", page);
  eq1kSlider = new QSlider(Qt::Vertical, page);
  eq1kSlider->setRange(-100, 100);
  eq1kSlider->setSteps(1, 10);
  connect(eq1kSlider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq1k(int)));
  grid->addWidget(eq1kText, 1, 5);
  grid->addWidget(eq1kSlider, 0, 5);

  QLabel* eq2kText = new QLabel("2kHz", page);
  eq2kSlider = new QSlider(Qt::Vertical, page);
  eq2kSlider->setRange(-100, 100);
  eq2kSlider->setSteps(1, 10);
  connect(eq2kSlider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq2k(int)));
  grid->addWidget(eq2kText, 1, 6);
  grid->addWidget(eq2kSlider, 0, 6);

  QLabel* eq4kText = new QLabel("4kHz", page);
  eq4kSlider = new QSlider(Qt::Vertical, page);
  eq4kSlider->setRange(-100, 100);
  eq4kSlider->setSteps(1, 10);
  connect(eq4kSlider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq4k(int)));
  grid->addWidget(eq4kText, 1, 7);
  grid->addWidget(eq4kSlider, 0, 7);

  QLabel* eq8kText = new QLabel("8kHz", page);
  eq8kSlider = new QSlider(Qt::Vertical, page);
  eq8kSlider->setRange(-100, 100);
  eq8kSlider->setSteps(1, 10);
  connect(eq8kSlider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq8k(int)));
  grid->addWidget(eq8kText, 1, 8);
  grid->addWidget(eq8kSlider, 0, 8);

  QLabel* eq16kText = new QLabel("16kHz", page);
  eq16kSlider = new QSlider(Qt::Vertical, page);
  eq16kSlider->setRange(-100, 100);
  eq16kSlider->setSteps(1, 10);
  connect(eq16kSlider, SIGNAL(valueChanged(int)), this, SIGNAL(signalNewEq16k(int)));
  grid->addWidget(eq16kText, 1, 9);
  grid->addWidget(eq16kSlider, 0, 9);

  KAccel* accel = new KAccel( this );
  accel->insert( "hide equalizer win", Qt::Key_E, this, SLOT(hide()) );


  connect(this, SIGNAL(defaultClicked()), this, SLOT(slotSetDefaultValues()));
}


Equalizer::~Equalizer()
{
}


void Equalizer::slotSetDefaultValues()
{
  eq30Slider->setValue(0);
  eq60Slider->setValue(0);
  eq125Slider->setValue(0);
  eq250Slider->setValue(0);
  eq500Slider->setValue(0);
  eq1kSlider->setValue(0);
  eq2kSlider->setValue(0);
  eq4kSlider->setValue(0);
  eq8kSlider->setValue(0);
  eq16kSlider->setValue(0);
}


/******* read from config-file ********/
void Equalizer::ReadValues(KConfig* config)
{
  config->setGroup("Equalizer");

  eq30Slider->setValue( config->readNumEntry("30Hz", 0) );
  eq60Slider->setValue( config->readNumEntry("60Hz", 0) );
  eq125Slider->setValue( config->readNumEntry("125Hz", 0) );
  eq250Slider->setValue( config->readNumEntry("250Hz", 0) );
  eq500Slider->setValue( config->readNumEntry("500Hz", 0) );
  eq1kSlider->setValue( config->readNumEntry("1kHz", 0) );
  eq2kSlider->setValue( config->readNumEntry("2kHz", 0) );
  eq4kSlider->setValue( config->readNumEntry("4kHz", 0) );
  eq8kSlider->setValue( config->readNumEntry("8kHz", 0) );
  eq16kSlider->setValue( config->readNumEntry("16kHz", 0) );
}


/************** save in config file *************/
void Equalizer::SaveValues(KConfig* config)
{
  config->setGroup("Equalizer");

  config->writeEntry("30Hz", eq30Slider->value());
  config->writeEntry("60Hz", eq60Slider->value());
  config->writeEntry("125Hz", eq125Slider->value());
  config->writeEntry("250Hz", eq250Slider->value());
  config->writeEntry("500Hz", eq500Slider->value());
  config->writeEntry("1kHz", eq1kSlider->value());
  config->writeEntry("2kHz", eq2kSlider->value());
  config->writeEntry("4kHz", eq4kSlider->value());
  config->writeEntry("8kHz", eq8kSlider->value());
  config->writeEntry("16kHz", eq16kSlider->value());
}  

