/***************************************************************************
                          kaffeine.cpp  - main widget
                             -------------------
    begin                : Don Apr 17 11:30:44 CEST 2003
    revision             : $Revision: 1.37 $
    last modified        : $Date: 2004/01/11 12:21:23 $ by $Author: juergenk $
    copyright            : (C) 2003-2004 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include <kiconloader.h>
#include <kurlrequesterdlg.h>
#include <kfiledialog.h>
#include <kurldrag.h>
#include <kmessagebox.h>
#include <kprocess.h>
#include <kdebug.h>

#include <qapplication.h>
#include <qinputdialog.h>

#include "screenshotpreview.h"
#include "kaffeine.h"
#include "kaffeine.moc"


KMP::KMP(QWidget *parent, const char *name, KCmdLineArgs* args) : QWidget(parent, name), DCOPObject("Kaffeine"),
                                          playList(NULL),cPanel(NULL), tRow(NULL), osDisplay(NULL),
                                          configWin(NULL), videoSettings(NULL), equalizer(NULL), systemTray(NULL), fsPanel(NULL), filterDialog(NULL),
                                          subtitleFile(QString::null), subtitleFileToUse(QString::null), currentVisualPlugin(QString::null),
                                          playSingleUrl(false), cUrl(QString::null), cTitle(QString::null), bmenu(NULL)
                                      
{
  cmdLineArgs = args;

  QString version = VERSION;
  kdDebug(555) << "Starting Kaffeine " << version << "...\n";
  kdDebug(555) << "http://kaffeine.sourceforge.net\n";
 
  setCaption("Kaffeine Media Player " + version);
       
  QVBoxLayout* boxLayout = new QVBoxLayout(this);
  
  videoWin = new VideoWindow(this, "videowin", cmdLineArgs->getOption("audiodriver"), cmdLineArgs->getOption("videodriver"));
  
  cPanel = new ControlPanel(this, "controlpanel");
  tRow = new TextRow(this, "inforow");


/******************* first run? ************************/

  QDir homedir = QDir::home();
  if (!homedir.exists(".kaffeine"))
    {
      kdDebug(555) << "Creating /.kaffeine directory\n";
      homedir.mkdir(".kaffeine");
    }

 QString stamp =  QDir::homeDirPath()+"/.kaffeine/mozilla-stamp";
 QString pluginpath = QDir::homeDirPath()+"/.mozilla/plugins";
    
/*  link the mozilla plugin with $HOME/.mozilla/plugins */

 if ( (!QFile::exists( stamp ))  && (!QFile::exists( pluginpath+"/kaffeineplugin.so" )) )
 {
   KProcess process;

   if (KMessageBox::questionYesNo(0, i18n("Should the Mozilla-Plugin be linked with ") + pluginpath + " ?") == KMessageBox::Yes)
   {
     QString installedPlugin = KAFFEINE_PLUGINDIR;
     installedPlugin.append("/kaffeineplugin.so");
    
     if (QFile::exists( installedPlugin ))
     {
        kdDebug(555) << "Mozilla-Plugin found: " << installedPlugin << endl;
        QDir plugindir( QDir::homeDirPath()+"/.mozilla");  
        plugindir.mkdir("plugins");   /* for sure */
        process << "ln" << "-s" << installedPlugin << pluginpath+"/kaffeineplugin.so";
        process.start(KProcess::Block, KProcess::Stderr);
        process.clearArguments();
     }
     else
       kdWarning(555) << "Mozilla-Plugin not installed correctly!" << endl;
   }

   process << "touch" << stamp;
   process.start(KProcess::Block, KProcess::Stderr);
 }
  
/******/

  osDisplay = new SplashWidget(0, "osd");
  equalizer = new Equalizer(0, "equalizer");

  InitActions();
  InitDVDMenu();
  InitMenu();
  InitContextMenu();
  InitSystemTray();
  InitFullscreenPanel();

  InitConnections();

  boxLayout->addWidget(menuBar);
  boxLayout->addWidget(videoWin);
  boxLayout->addWidget(tRow);
  boxLayout->addWidget(cPanel);

  setAcceptDrops(true);
}


KMP::~KMP()
{
  delete osDisplay;
  delete equalizer;
  delete videoSettings;
  delete playList;
  delete fsPanel;
  delete filterDialog;
}


/***********************************************
 *  FINALIZE THE GUI
 ***********************************************/

void KMP::polish()
{

  if (!(videoWin->xineRunning) || (playList)) return;
  
/*** autoplay plugins ***/

  KAction* autoPlayPlugin = NULL;
  QStringList autoPlayList;
  uint i = 0;
  QSignalMapper* mapper = new QSignalMapper(this);
  connect(mapper, SIGNAL(mapped(const QString&)), this, SLOT(slotPlayAutoplayPlugin(const QString&)));
  
  videoWin->GetAutoplayPlugins(autoPlayList);

  while(i<autoPlayList.count())
  {
   autoPlayPlugin = new KAction(autoPlayList[i], "cdrom_unmount", 0, mapper, SLOT(map()), actionCollection);
   autoPlayPlugin->plug(playMenu);
   mapper->setMapping(autoPlayPlugin, autoPlayList[i]);
   i++;
   autoPlayPlugin->setWhatsThis(autoPlayList[i]);
   i++;
  }

/*** visual plugins ***/

  QStringList visuals;
  visuals << "none";
  visualMenu->setCheckable(true);

  videoWin->GetVisualPlugins(visuals);

  for(i = 0; i < visuals.count(); i++)
   visualMenu->insertItem( visuals[i], i );
    
  connect(visualMenu, SIGNAL(activated(int)), this, SLOT(slotSetVisual(int)));
  
/***/  
  cPanel->setVolume(videoWin->GetVolume());

/** playlist  **/

  playList = new PlayList(0, "playlist");
  playList->SetAutoplayList(autoPlayList);

/**** video settings ****/
  
  int hue, sat, contrast, bright, audioAmp, avOffset, spuOffset;

  videoWin->GetVideoSettings(hue, sat, contrast, bright, audioAmp, avOffset, spuOffset);
  
  videoSettings = new VideoSettings(hue, sat, contrast, bright, audioAmp, avOffset,
                                    spuOffset, 0, "videosettings");


/***** postprocessing *******/

  QStringList filterList;
  videoWin->GetPostprocessFilterNames( filterList );

  filterDialog = new FilterDialog( filterList, 0, "filterconfigdialog" );
                                    

/*** connections with playlist, videosettings and filterdialog ***/

  connect(filterDialog, SIGNAL(signalCreateFilter(const QString&, QWidget*)), videoWin, SLOT(slotCreateFilter(const QString&, QWidget*)));            
  connect(filterDialog, SIGNAL(signalRemoveAll()), videoWin, SLOT(slotRemoveAllFilters()));

  connect(playList, SIGNAL(signalPlayDirect()), this, SLOT(slotPlay()));
  connect(videoWin, SIGNAL(signalNewMrlReference(const QString&)), playList, SLOT(slotAddMrl(const QString&)));
  connect(videoWin, SIGNAL(signalLengthInfo(const QString&)),
          playList, SLOT(slotGetLengthInfo(const QString&)));
  connect(videoWin, SIGNAL(signalMetaInfo(const QString&)), playList, SLOT(slotGetMetaInfo(const QString&)));
  connect(videoWin, SIGNAL(signalStreamInfo(const QString&)), playList, SLOT(slotGetStreamInfo(const QString&)));
  connect(playList, SIGNAL(signalQuit()), this, SLOT(slotQuit()));
  connect(playList, SIGNAL(signalAutoplayPlugin(const QString&, QListViewItem*)), this, SLOT(slotAddAutoplayUrls(const QString&, QListViewItem*)));
  connect(playList, SIGNAL(signalPlayPause()), cPanel, SLOT(slotPlayPause()));
  connect(playList, SIGNAL(signalStop()), this, SLOT(slotStop()));
          
  connect(videoSettings, SIGNAL(signalNewHue(int)), videoWin, SLOT(slotSetHue(int)));
  connect(videoSettings, SIGNAL(signalNewSaturation(int)), videoWin, SLOT(slotSetSaturation(int)));
  connect(videoSettings, SIGNAL(signalNewContrast(int)), videoWin, SLOT(slotSetContrast(int)));
  connect(videoSettings, SIGNAL(signalNewBrightness(int)), videoWin, SLOT(slotSetBrightness(int)));
  connect(videoSettings, SIGNAL(signalNewAudioAmp(int)), videoWin, SLOT(slotSetAudioAmp(int)));
  connect(videoSettings, SIGNAL(signalNewAVOffset(int)), videoWin, SLOT(slotSetAVOffset(int)));
  connect(videoSettings, SIGNAL(signalNewSpuOffset(int)), videoWin, SLOT(slotSetSpuOffset(int)));
  
  tRow->slotChangeText(QString(i18n("Ready.")));

/*** config stuff ***/

  config = kapp->config();
  ReadConfig();

/*** parse command line **/

  KURL::List files;

  if (cmdLineArgs->count() > 0)
  {
    for (int i = 0; i<cmdLineArgs->count(); i++)
    {
      files.append(cmdLineArgs->url(i));
    }
  }

 CmdLineArgs( files, cmdLineArgs->getOption("device"), cmdLineArgs->isSet("play"), cmdLineArgs->isSet("fullscreen"), cmdLineArgs->isSet("minimal"), cmdLineArgs->isSet("enqueue"), cmdLineArgs->isSet("verbose") );

}


void KMP::SaveConfig()
{
  QFont f;
  QColor c;
  QString mstr;
 
// kdDebug(555) << "Save config\n";

  config->setGroup("General Options");

  if (!viewFullscreen->isChecked())
    config->writeEntry("KMP Geometry", size());
  
  config->writeEntry("Minimal Mode", viewMinimalMode->isChecked());
  config->writeEntry("PlayList Geometry", playList->size());
  config->writeEntry("PlayList visible", playList->isVisible());
  config->writeEntry("VideoSettings Geometry", videoSettings->size());
  config->writeEntry("Filter Dialog Geometry", filterDialog->size());
  config->writeEntry("Endless Mode", playEndlessPlay->isChecked());
  config->writeEntry("Random Mode", playRandomPlay->isChecked());

  config->writeEntry("Load Playlist", loadPlaylistAtStartup);
  config->writeEntry("Current Playlist:", playList->GetLastPlaylist());

  config->writeEntry("Meta String", videoWin->GetMetaString());
  config->writeEntry("Screensaver Timeout", videoWin->GetScreensaverTimeout());

  tRow->GetConfig(f, c);
  config->writeEntry("Textrow Font", f);
  config->writeEntry("Textrow Color", c);

  config->setGroup("OSD Options");
  bool u, a;
  QFont font;
  QColor color;
  int x, y, s ,t;
  osDisplay->GetConfig(u, a, s, font, color, t, x, y);
  config->writeEntry("Use OSD", u);
  config->writeEntry("Animate", a);
  config->writeEntry("Animation Speed",s );
  config->writeEntry("OSD Font", font);
  config->writeEntry("OSD Color", color);
  config->writeEntry("Display Time", t);
  config->writeEntry("X-Pos", x);
  config->writeEntry("y-Pos", y);

  config->setGroup("Broadcasting Options");
  config->writeEntry("Port", broadcastPort);
  config->writeEntry("Master Address", broadcastMasterAddress);

  config->setGroup("Visualization");
  config->writeEntry("Visual Plugin", currentVisualPlugin);
  
  playList->SaveToolbarSettings(config);
  
  equalizer->SaveValues(config);

  fileOpenRecent->saveEntries(config, "Recent Files");
}


void KMP::ReadConfig()
{
  QSize size;
  QFont f;
  QColor c;
  bool b;

//  kdDebug(555) << "Read config\n";

  config->setGroup("General Options");
  size = QSize(450, 420);
  size = config->readSizeEntry("KMP Geometry", &size);
  resize(size);

  b = config->readBoolEntry("Minimal Mode", false);
  if (b)
  {
    cPanel->hide();
    menuBar->hide();
    viewMinimalMode->setChecked(true);
  }  

  size = QSize(350,350);
  size = config->readSizeEntry("PlayList Geometry", &size);
  playList->resize(size);
  b = config->readBoolEntry("PlayList visible", false);
  if (b) playList->show();

  size = QSize(350,100);
  size = config->readSizeEntry("VideoSettings Geometry", &size);
  videoSettings->resize(size);

  size = QSize(500,500);
  size = config->readSizeEntry("Filter Dialog Geometry", &size);
  filterDialog->resize(size);
  
  b = config->readBoolEntry("Endless Mode", false);
  playEndlessPlay->setChecked(b);
  playList->SetEndless(b);
     
  b = config->readBoolEntry("Random Mode", false);
  playRandomPlay->setChecked(b);
  playList->SetRandomPlay(b);

  loadPlaylistAtStartup = config->readBoolEntry("Load Playlist", true);
  QString pl = config->readEntry("Current Playlist:", QString());
  playList->SetLastPlaylist(pl);
  
  int ssTimeout = config->readNumEntry("Screensaver Timeout", 2);
  QString metaStr = config->readEntry("Meta String", QString("%1 - %2 (%3)"));
  playList->slotSetMetaInfoString(metaStr);
  videoWin->slotSetConfig(metaStr, ssTimeout);

  f = KGlobalSettings::generalFont();
  QFont infoFont = config->readFontEntry("Textrow Font", &f);
  c = KGlobalSettings::baseColor();
  QColor infoColor = config->readColorEntry("Textrow Color", &c);
  tRow->slotSetConfig(&infoFont, &infoColor);

  config->setGroup("OSD Options");
  bool osdUse, osdAnim;
  QFont osdFont;
  QColor osdColor;
  int osdX, osdY, osdAnimSpeed, osdTime;
  
  osdUse = config->readBoolEntry("Use OSD", true);
  osdAnim = config->readBoolEntry("Animate", true);
  osdAnimSpeed = config->readNumEntry("Animation Speed", 15 );
  f.setPointSize(18);
  osdFont = config->readFontEntry("OSD Font", &f);
  c = QColor(0xFF, 0x89, 0);
  osdColor = config->readColorEntry("OSD Color", &c);
  osdTime = config->readNumEntry("Display Time", 2);
  osdX = config->readNumEntry("X-Pos", 5);
  osdY = config->readNumEntry("y-Pos", 5);

  config->setGroup("Broadcasting Options");
  broadcastPort = config->readNumEntry("Port", 8080);
  broadcastMasterAddress = config->readEntry("Master Address", "localhost");

  config->setGroup("Visualization");
  currentVisualPlugin = config->readEntry("Visual Plugin", "goom");
  for (uint i = 0; i < visualMenu->count(); i++)
   if ( visualMenu->text( i ) == currentVisualPlugin )
   {
     slotSetVisual( i );
     break;
   }  
  
  playList->LoadToolbarSettings(config);
  
  equalizer->ReadValues(config);
  
  fileOpenRecent->loadEntries(config, "Recent Files");

  osDisplay->slotSetConfig(osdUse, osdAnim, osdAnimSpeed,  &osdFont, &osdColor,
                           osdTime, osdX, osdY);

  configWin = new ConfigWidget(loadPlaylistAtStartup, ssTimeout, &metaStr,
                  &infoFont, &infoColor,osdUse, osdAnim, osdAnimSpeed,
                  &osdFont, &osdColor, osdTime, osdX, osdY);


 connect(configWin, SIGNAL(signalNewGenConfig(bool)), this, SLOT(slotGenConfig(bool)));
 connect(configWin, SIGNAL(signalNewInfoRowConfig(QFont*, QColor*)), tRow, SLOT(slotSetConfig(QFont*, QColor*)));
 connect(configWin, SIGNAL(signalNewVideoConfig(const QString&, int)),
                    videoWin, SLOT(slotSetConfig(const QString&, int)));
 connect(configWin, SIGNAL(signalNewOsdConfig(bool, bool, int, QFont*, QColor*, int, int, int)),
         osDisplay, SLOT(slotSetConfig(bool, bool, int, QFont*, QColor*, int, int, int)));
 connect(configWin, SIGNAL(signalMSForPlaylist(const QString&)), playList, SLOT(slotSetMetaInfoString(const QString&)));                                           
}



/************************************************
 *          INITIALIZIATON
 ************************************************/

void KMP::InitConnections()
{
 
  connect(cPanel, SIGNAL(signalPlay()), this, SLOT(slotPlay()));
  connect(cPanel, SIGNAL(signalVolumeChanged(int)), videoWin, SLOT(slotSetVolume(int)));
  connect(videoWin, SIGNAL(signalNewInfo(const QString&)), tRow, SLOT(slotChangeText(const QString&)));
  connect(videoWin, SIGNAL(signalNewTitle(const QString&)), this, SLOT(slotGetNewTitle(const QString&)));
  connect(cPanel, SIGNAL(signalPlayNext()), this, SLOT(slotPlayNext()));
  connect(videoWin, SIGNAL(signalPlaybackFinished()), this, SLOT(slotEndOfPlayback()));
  connect(cPanel, SIGNAL(signalPlayPrevious()), this, SLOT(slotPlayPrev()));
  connect(cPanel, SIGNAL(signalStop()), this , SLOT(slotStop()));
  connect(videoWin, SIGNAL(signalHasChapters(bool)), this, SLOT(slotHasChapters(bool)));
  connect(cPanel, SIGNAL(signalPauseOn()), videoWin, SLOT(slotSpeedPause()));
  connect(cPanel, SIGNAL(signalPauseOff()), videoWin, SLOT(slotSpeedNormal()));
  connect(videoWin, SIGNAL(signalNewChannels(const QStringList&, const QStringList&, int, int)),
          cPanel, SLOT(slotNewChannels(const QStringList&, const QStringList&, int ,int)));
  connect(videoWin, SIGNAL(signalNewChannels(const QStringList&, const QStringList&, int, int)),
          this, SLOT(slotNewChannels(const QStringList&, const QStringList&, int ,int)));        
  connect(cPanel, SIGNAL(signalSetAudioChannel(int)), videoWin, SLOT(slotSetAudioChannel(int)));
  connect(cPanel, SIGNAL(signalSetSubtitleChannel(int)), videoWin, SLOT(slotSetSubtitleChannel(int)));        
  connect(videoWin, SIGNAL(signalNewPosition(int, const QString&)),
          cPanel, SLOT(slotNewPosition(int, const QString&)));
  connect(cPanel, SIGNAL(signalChangePosition(int)), videoWin, SLOT(slotChangePosition(int)));
  connect(cPanel, SIGNAL(signalTogglePlayMode()), videoWin, SLOT(slotTogglePlayMode()));
  connect(cPanel, SIGNAL(signalPlayList()), this, SLOT(slotShowPlayList()));
  connect(cPanel, SIGNAL(signalSaveStreamAs()), this, SLOT(slotSaveStreamAs()));
  
  connect(videoWin, SIGNAL(signalShowContextMenu(const QPoint&)), this, SLOT(slotShowContextMenu(const QPoint&)));
  connect(videoWin, SIGNAL(signalShowFullscreenPanel(const QPoint&)), this, SLOT(slotShowFullscreenPanel(const QPoint&)));
  connect(videoWin, SIGNAL(signalShowOSD(const QString&)), osDisplay, SLOT(slotOsdShow(const QString&)));
  connect(videoWin, SIGNAL(signalQuit()), this, SLOT(slotQuit()));
  connect(videoWin, SIGNAL(signalXineMessage(const QString&)), this, SLOT(slotXineMessage(const QString&)));
/** equalizer **/

  connect(equalizer, SIGNAL(signalNewEq30(int)), videoWin, SLOT(slotSetEq30(int)));
  connect(equalizer, SIGNAL(signalNewEq60(int)), videoWin, SLOT(slotSetEq60(int)));
  connect(equalizer, SIGNAL(signalNewEq125(int)), videoWin, SLOT(slotSetEq125(int)));
  connect(equalizer, SIGNAL(signalNewEq250(int)), videoWin, SLOT(slotSetEq250(int)));
  connect(equalizer, SIGNAL(signalNewEq500(int)), videoWin, SLOT(slotSetEq500(int)));
  connect(equalizer, SIGNAL(signalNewEq1k(int)), videoWin, SLOT(slotSetEq1k(int)));
  connect(equalizer, SIGNAL(signalNewEq2k(int)), videoWin, SLOT(slotSetEq2k(int)));
  connect(equalizer, SIGNAL(signalNewEq4k(int)), videoWin, SLOT(slotSetEq4k(int)));
  connect(equalizer, SIGNAL(signalNewEq8k(int)), videoWin, SLOT(slotSetEq8k(int)));
  connect(equalizer, SIGNAL(signalNewEq16k(int)), videoWin, SLOT(slotSetEq16k(int)));

 /** fullscreen panel **/
  connect(videoWin, SIGNAL(signalNewPosition(int, const QString&)), fsPanel, SLOT(slotSetPosition(int, const QString&)));
  connect(videoWin, SIGNAL(signalLengthInfo(const QString&)), fsPanel, SLOT(slotSetLength(const QString&)));
  connect(videoWin, SIGNAL(signalMetaInfo(const QString&)), fsPanel, SLOT(slotSetTitle(const QString&)));
  connect(videoWin, SIGNAL(signalStreamInfo(const QString&)), fsPanel, SLOT(slotSetStreamInfo(const QString&)));

  connect(fsPanel, SIGNAL(signalPlayPause()), cPanel, SLOT(slotPlayPause()));
  connect(fsPanel, SIGNAL(signalNext()), this, SLOT(slotPlayNext()));
  connect(fsPanel, SIGNAL(signalPrevious()), this, SLOT(slotPlayPrev()));
  connect(fsPanel, SIGNAL(signalStop()), this, SLOT(slotStop()));
  connect(fsPanel, SIGNAL(signalRewind()), cPanel, SLOT(slotPosMinus()));
  connect(fsPanel, SIGNAL(signalForward()), cPanel, SLOT(slotPosPlus()));
  connect(fsPanel, SIGNAL(signalVolUp()), cPanel->volSlider, SLOT(subtractStep()));
  connect(fsPanel, SIGNAL(signalVolDown()), cPanel->volSlider, SLOT(addStep()));
  connect(fsPanel, SIGNAL(signalDVDMenu()), videoWin, SLOT(slotMenu1()));
}
  

void KMP::InitActions()
{

 actionCollection = new KActionCollection(this);

 fileOpen = KStdAction::open(this, SLOT(slotOpenFile()), actionCollection);
 fileOpenUrl= new KAction(i18n("Open &URL"), "package_network", Qt::CTRL|Qt::Key_U, this, SLOT(slotOpenUrl()), actionCollection, "openurl");
 fileOpenSubtitleFile = new KAction(i18n("Open Sub&title File"), "fileopen", Qt::Key_T, this, SLOT(slotOpenSubtitleFile()), actionCollection, "opensubtitlefile");
 fileOpenRecent = KStdAction::openRecent(this, SLOT(slotPlayRecent(const KURL&)), actionCollection);
 fileSaveScreenshot = new KAction(i18n("&Save Screenshot"), "filesave", Qt::CTRL|Qt::Key_S, this, SLOT(slotScreenshot()), actionCollection, "screenshot");
 //fileEject = new KAction(i18n("&Eject"), "player_eject", 0, videoWin, SLOT(slotEject()), actionCollection, "eject");
 fileQuit= KStdAction::quit(this, SLOT(slotQuit()), actionCollection);
 
 playUseSubtitleFile = new KToggleAction(i18n("Use Sub&title File"), 0, Qt::CTRL|Qt::Key_T, this, SLOT(slotUseSubtitleFile()), actionCollection, "usesubtitlefile");
 playEndlessPlay = new KToggleAction(i18n("&Endless Play"), 0 , Qt::CTRL|Qt::Key_E, this, SLOT(slotEndlessMode()), actionCollection,"endlessplay");
 playRandomPlay = new KToggleAction(i18n("&Random Play"), 0 , Qt::CTRL|Qt::Key_R, this, SLOT(slotRandomPlay()), actionCollection,"randomplay");

 broadcastMaster = new KToggleAction(i18n("&Define As Master"), 0, 0, this, SLOT(slotToggleBroadcastMaster()), actionCollection, "broadcastmaster");
 broadcastSlave  = new KAction(i18n("&Play As Slave"), "network", 0, this, SLOT(slotBroadcastSlave()), actionCollection, "broadcastslave");
 
 viewFullscreen= new KToggleAction(i18n("&Fullscreen"), NULL, Qt::Key_F, this, SLOT(slotToggleFullscreen()), actionCollection, "fullscreen");
 viewMinimalMode = new KToggleAction(i18n("&Minimal Mode"), NULL, Qt::Key_M, this, SLOT(slotToggleMinimalMode()), actionCollection, "minimalmode");

 viewDeinterlace = new KToggleAction(i18n("&Deinterlace"), 0, Qt::Key_I, videoWin, SLOT(slotToggleDeinterlace()), actionCollection, "deinterlace");
 viewDeinterlace->setWhatsThis(i18n("Activate this for interlaced streams, some DVD's for example."));
 viewPostprocess = new KToggleAction(i18n("Enable Video &Postprocessing"), 0, Qt::CTRL|Qt::Key_X, this, SLOT(slotEnableFilters()), actionCollection, "enablepostprocessing");
 viewPostprocess->setChecked( true );
 
 viewZoomIn = KStdAction::zoomIn(videoWin, SLOT(slotZoomIn()), actionCollection);
 viewZoomOut = KStdAction::zoomOut(videoWin, SLOT(slotZoomOut()), actionCollection);
 viewZoomOff = KStdAction::fitToPage(videoWin, SLOT(slotZoomOff()), actionCollection);
 viewToggleTv = new KToggleAction(i18n("Toggle T&V-Mode"),"tv", Qt::ALT|Qt::Key_T,this, SLOT(slotToggleTv()),actionCollection, "toggletv");

 aspectRatioAuto =new KAction(i18n("&auto"), "viewmagfit", Qt::Key_F5, videoWin, SLOT(slotAspectRatioAuto()), actionCollection, "aspectratioauto");
 aspectRatio4_3 = new KAction(i18n("&4:3"), "viewmagfit", Qt::Key_F6, videoWin, SLOT(slotAspectRatio4_3()), actionCollection, "aspectratio43");
 aspectRatio16_9 =new KAction(i18n("&16:9"), "viewmagfit", Qt::Key_F7, videoWin, SLOT(slotAspectRatio16_9()), actionCollection, "aspectratio169");
 aspectRatioSquare = new KAction(i18n("&1:1"), "viewmagfit", Qt::Key_F8, videoWin, SLOT(slotAspectRatioSquare()), actionCollection, "aspectratio12");
 
 configVideoControl= new KAction(i18n("&Video Settings"), "configure", Qt::Key_V, this, SLOT(slotVideoSettings()), actionCollection, "videocontrol");
 configEqualizer = new KAction(i18n("&Equalizer"), "configure", Qt::Key_E, this, SLOT(slotEqualizer()), actionCollection, "equalizer");
 configPostFilter = new KAction(i18n("Video &Postprocessing Filters"), "filter", Qt::Key_X, this, SLOT(slotShowFilterConfig()), actionCollection, "filterconfig");
 configXine = new KAction(i18n("Configure &xine"), "edit", 0, this, SLOT(slotConfigXine()), actionCollection, "configxine");
 configKMP = KStdAction::preferences(this, SLOT(slotShowSetup()), actionCollection);

 viewToggleTv->setEnabled(false);

/** actions for context menu **/

 controlPlayPause = new KAction(i18n("Play/Pause"), "player_play", Qt::Key_Space, cPanel, SLOT(slotPlayPause()), actionCollection, "controlplaypause");
 controlNext      = new KAction(i18n("Next Entry/Chapter"), "player_end", Qt::Key_Down, this, SLOT(slotPlayNext()), actionCollection, "controlnext");
 controlPrevious  = new KAction(i18n("Previous Entry/Chapter"), "player_start", Qt::Key_Up, this, SLOT(slotPlayPrev()), actionCollection, "controlprevious");
 controlStop      = new KAction(i18n("Stop"), "player_stop", Qt::Key_Backspace, this, SLOT(slotStop()), actionCollection, "controlstop");
 controlShowHidePlaylist = new KAction(i18n("Playlist"), "player_eject", Qt::Key_P, this, SLOT(slotShowPlayList()), actionCollection, "showhideplaylist");
 controlSaveStreamAs = new KAction(i18n("Save Stream As..."),"player_record", Qt::Key_R, this, SLOT(slotSaveStreamAs()), actionCollection, "savestreamas");
} 


void KMP::InitMenu()
{

 KAboutApplication* kabout = new KAboutApplication(this);
 KHelpMenu* khelp = new KHelpMenu(this);
 connect(khelp, SIGNAL(showAboutApplication()), kabout, SLOT(show()));

 menuBar = new KMenuBar(this);
 menuBar->setSizePolicy ( QSizePolicy (QSizePolicy::MinimumExpanding, QSizePolicy::Fixed) );

/** init bookmark menu **/
 bookmarkMenu = new KPopupMenu(this);
 bookmarkManager = KBookmarkManager::managerForFile(QDir::homeDirPath() + "/.kaffeine/bookmarks.xml");
 bmenu = new KBookmarkMenu(bookmarkManager, this, bookmarkMenu, actionCollection, true);

 fileMenu = new KPopupMenu(this);
 playMenu = new KPopupMenu(this);
 visualMenu = new KPopupMenu(this);
 broadcastMenu = new KPopupMenu(this);
 viewMenu = new KPopupMenu(this);
 aspectMenu = new KPopupMenu(this);
 configMenu = new KPopupMenu(this);
 helpMenu = khelp->menu();

 fileOpen->plug(fileMenu);
 fileOpenUrl->plug(fileMenu);
 fileOpenSubtitleFile->plug(fileMenu);
 fileOpenRecent->plug(fileMenu);
 fileMenu->insertSeparator();
 fileSaveScreenshot->plug(fileMenu);
 fileMenu->insertSeparator();
// fileEject->plug(fileMenu);
// fileMenu->insertSeparator();
 fileQuit->plug(fileMenu);

 broadcastMaster->plug(broadcastMenu);
 broadcastSlave->plug(broadcastMenu);

 playEndlessPlay->plug(playMenu);
 playRandomPlay->plug(playMenu);
 playMenu->insertSeparator();
 playUseSubtitleFile->plug(playMenu);
 playMenu->insertSeparator();
 playMenu->insertItem( i18n("&Network Broadcasting"), broadcastMenu);
 playMenu->insertSeparator();
 playMenu->insertItem( i18n("&DVD Menus"), dvdMenu);
 playMenu->insertSeparator();

 aspectRatioAuto->plug(aspectMenu);
 aspectRatio4_3->plug(aspectMenu);
 aspectRatio16_9->plug(aspectMenu);
 aspectRatioSquare->plug(aspectMenu);

 viewFullscreen->plug(viewMenu);
 viewMinimalMode->plug(viewMenu);
 viewMenu->insertSeparator();
 viewMenu->insertItem(i18n("Audio &Visualization"), visualMenu);
 viewPostprocess->plug(viewMenu);
 viewDeinterlace->plug(viewMenu);
  viewMenu->insertSeparator();
 viewMenu->insertItem(i18n("&Aspect Ratio"), aspectMenu);
 viewMenu->insertSeparator();
 viewZoomOff->plug(viewMenu);
 viewZoomIn->plug(viewMenu);
 viewZoomOut->plug(viewMenu);

// viewMenu->insertSeparator();
// viewToggleTv->plug(viewMenu);

 configVideoControl->plug(configMenu);
 configEqualizer->plug(configMenu);
 configPostFilter->plug(configMenu);
 configMenu->insertSeparator();
 configXine->plug(configMenu);
 configMenu->insertSeparator();
 configKMP->plug(configMenu);

 menuBar->insertItem(i18n("&File"), fileMenu);
 menuBar->insertItem(i18n("&Play"), playMenu);
 menuBar->insertItem(i18n("&View"), viewMenu);
 menuBar->insertItem(i18n("&Bookmarks"), bookmarkMenu);
 menuBar->insertItem(i18n("&Config"), configMenu);
 menuBar->insertItem(i18n("&Help"), helpMenu);
}

/******** system tray ***************/

void KMP::InitSystemTray()
{
  systemTray = new SystemTray(this, "systemtray");

  connect(systemTray, SIGNAL(signalPlayPause()), cPanel, SLOT(slotPlayPause()));
  connect(systemTray, SIGNAL(signalNext()), this, SLOT(slotPlayNext()));
  connect(systemTray, SIGNAL(signalPrev()), this, SLOT(slotPlayPrev()));
  connect(systemTray, SIGNAL(signalStop()), this, SLOT(slotStop()));
  connect(systemTray, SIGNAL(signalPlayFiles(const KURL::List&)), this, SLOT(slotPlayFiles(const KURL::List&)));
  connect(cPanel, SIGNAL(signalPauseOn()), systemTray, SLOT(slotPauseIcon()));
  connect(cPanel, SIGNAL(signalPauseOff()), systemTray, SLOT(slotPlayIcon()));
  connect(systemTray, SIGNAL(signalShowPlaylist()), this, SLOT(slotShowPlayList()));
  connect(systemTray, SIGNAL(signalShowEqualizer()), this, SLOT(slotEqualizer()));
  connect(systemTray, SIGNAL(signalShowVideosettings()), this, SLOT(slotVideoSettings()));
  connect(systemTray, SIGNAL(signalSaveStreamAs()), this, SLOT(slotSaveStreamAs()));
}  


/**** context menu ********/

void KMP::InitContextMenu()
{
 contextMenu = new KPopupMenu(this);

 controlMenu = new KPopupMenu(this);
 audioChannelMenu = new KPopupMenu(this);
 audioChannelMenu->setCheckable(true);
 subtitleChannelMenu = new KPopupMenu(this);
 subtitleChannelMenu->setCheckable(true);
                
 controlSaveStreamAs->plug(controlMenu);
 controlPlayPause->plug(controlMenu);
 controlStop->plug(controlMenu);
 controlNext->plug(controlMenu);
 controlPrevious->plug(controlMenu);

 controlMenu->insertSeparator();
 controlMenu->insertItem(i18n("Audiochannel"), audioChannelMenu);
 controlMenu->insertItem(i18n("Subtitles"), subtitleChannelMenu);
 
 contextMenu->insertItem(i18n("&Control"), controlMenu);
 controlShowHidePlaylist->plug(contextMenu);
 contextMenu->insertSeparator();
 contextMenu->insertItem(i18n("&File"), fileMenu);
 contextMenu->insertItem(i18n("&Play"), playMenu);
 contextMenu->insertItem(i18n("&View"), viewMenu);
 contextMenu->insertItem(i18n("&Bookmarks"), bookmarkMenu);
 contextMenu->insertItem(i18n("&Config"), configMenu);
 contextMenu->insertItem(i18n("&Help"), helpMenu);

 connect(audioChannelMenu, SIGNAL(activated(int)), this, SLOT(slotContextMenuAudioChannel(int)));
 connect(subtitleChannelMenu, SIGNAL(activated(int)), this, SLOT(slotContextMenuSubtitleChannel(int)));
}


void KMP::InitDVDMenu()
{
  dvdMenu = new KPopupMenu(this);
  dvdMenu->insertItem( KGlobal::iconLoader()->loadIcon("view_detailed", KIcon::Small), i18n("&Menu Toggle"), videoWin, SLOT( slotMenu1() ), Qt::Key_D );
  dvdMenu->insertSeparator();
  dvdMenu->insertItem( i18n("&Title"), videoWin, SLOT( slotMenu2() ) );
  dvdMenu->insertItem( i18n("&Root"), videoWin, SLOT( slotMenu3() ) );
  dvdMenu->insertItem( i18n("&Subpicture"), videoWin, SLOT( slotMenu4() ) );
  dvdMenu->insertItem( i18n("&Audio"), videoWin, SLOT( slotMenu5() ) );
  dvdMenu->insertItem( i18n("An&gle"), videoWin, SLOT( slotMenu6() ) );
  dvdMenu->insertItem( i18n("&Part"), videoWin, SLOT( slotMenu7() ) );
}  


/*** visual plugins ***/

void KMP::slotSetVisual(int visId)
{
  currentVisualPlugin = visualMenu->text( visId );
 // kdDebug(555) << "Use visual plugin: " << currentVisualPlugin << endl;

  for (uint i = 0; i < visualMenu->count(); i++)
   if (visualMenu->isItemChecked(i))
     visualMenu->setItemChecked(i , false);
     
  visualMenu->setItemChecked( visId, true );

  videoWin->SetVisualPlugin( currentVisualPlugin );
}  
  

/******* fullscreen panel **************/

void KMP::InitFullscreenPanel()
{
  kdDebug(555) << "Init fullscreen panel\n";

  fsPanel = new FullscreenPanel(0, "fullscreenpanel");
  fsPanel->SetContextMenus( audioChannelMenu, subtitleChannelMenu );
  fsPanel->SetWidgetToActivate( this );
}  


void KMP::slotShowFullscreenPanel(const QPoint& pos)
{
  if ( (!viewFullscreen->isChecked()) && (!viewMinimalMode->isChecked()) ) return;
  if (contextMenu->isShown()) return;

  if (fsPanel->isShown())
    fsPanel->hide();
   else 
    fsPanel->ShowAt( pos );
}  


/******** context menu **********/

void KMP::slotShowContextMenu(const QPoint& w)
{
  /* show context menu only in minmal or fullscreen mode */
  if ( (!viewFullscreen->isChecked()) && (!viewMinimalMode->isChecked()) ) return;
  
  contextMenu->popup(w);
}


void KMP::slotNewChannels(const QStringList& audio, const QStringList& subtitles, int currentAudio, int currentSub)
{
  audioChannelMenu->clear();
  subtitleChannelMenu->clear();

  audioChannelMenu->insertItem(i18n("auto"), 0);
  for (uint a = 0; a<audio.count(); a++)
  {
    audioChannelMenu->insertItem( audio[a], a+1 );
  }

  audioChannelMenu->setItemChecked( currentAudio+1, true);

  subtitleChannelMenu->insertItem(i18n("off"), 0);
  for (uint s = 0; s<subtitles.count(); s++)
  {
    subtitleChannelMenu->insertItem( subtitles[s], s+1 );
  }

  subtitleChannelMenu->setItemChecked( currentSub+1, true);
}  


void KMP::slotContextMenuVolume(int id)
{  
  cPanel->volSlider->setValue( -(id*10) );
}


void KMP::slotContextMenuAudioChannel(int id)
{
  for (uint i = 0; i < audioChannelMenu->count(); i++)
   if (audioChannelMenu->isItemChecked(i))
     audioChannelMenu->setItemChecked(i , false);
  audioChannelMenu->setItemChecked( id, true );   

  cPanel->audioChannelCB->setCurrentItem(id);
  videoWin->slotSetAudioChannel(id);
}  


void KMP::slotContextMenuSubtitleChannel(int id)
{
  for (uint i = 0; i < subtitleChannelMenu->count(); i++)
   if (subtitleChannelMenu->isItemChecked(i))
     subtitleChannelMenu->setItemChecked(i , false);
  subtitleChannelMenu->setItemChecked( id, true );   

  cPanel->subtitleCB->setCurrentItem(id);
  videoWin->slotSetSubtitleChannel(id);
}   
  
/***********************/

void KMP::LoadPlaylist()
{
  if (loadPlaylistAtStartup)
  {
    kdDebug(555) << "Load Playlist\n";
    tRow->slotChangeText(i18n("Load Playlist..."));
    playList->ReloadLastPlaylist();
    tRow->slotChangeText(i18n("Ready."));
 }
}


void KMP::CmdLineArgs(const KURL::List& files, const QString device, bool playAtStartup, bool startFullscreen, bool minimalMode, bool enqueueFiles, bool verbose)
{
  if (verbose)
    videoWin->SetXineVerbosityDebug();

  if (minimalMode)
  {
    viewMinimalMode->setChecked(true);
    slotToggleMinimalMode();
  }  

  if (startFullscreen)
  {
    viewFullscreen->setChecked(true);
    slotToggleFullscreen();
  }

  if (device != "do_nothing")
  {
    videoWin->SetDevice(device);
  }  

  if (files.count() > 0)
  {
    if (!enqueueFiles)
    {
      playList->slotClearList();
    }  
    QListViewItem* last = playList->GetLast();
    playList->Add(files, last);
    if (last == NULL)  /* !enqueueFiles */
      QTimer::singleShot(200, this, SLOT(slotPlay()));
  }
  else
  {
   LoadPlaylist();
   
   if (playAtStartup)
     QTimer::singleShot(200, this, SLOT(slotPlay()));
    else
     videoWin->PlayLOGO();
  }  
}         


/******** execute remote commands (from command line or via DCOP) ***********/

void KMP::ExecuteRemoteCommand(const QString& cmd)
{
  if (cmd == "play")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    if (cPanel->isPlayButton)
      cPanel->slotPlayPause();
    return;
  }
  if (cmd == "pause")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    if (!cPanel->isPlayButton)
      cPanel->slotPlayPause();
    return;
  }
  if (cmd == "next")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    if (cPanel->isPlaying)
      slotPlayNext();
    return;  
  }    
  if (cmd == "prev")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    if (cPanel->isPlaying)
      slotPlayPrev();
    return;  
  }    
  if (cmd == "stop")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    if (cPanel->isPlaying)
      slotStop();
    return;  
  }    

  if (cmd == "volup")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    cPanel->volSlider->subtractStep();
    return;
  }  
  if (cmd == "voldown")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    cPanel->volSlider->addStep();
    return;
  }  
  if (cmd == "incpos")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    cPanel->slotPosPlus();
    return;
  }  
  if (cmd == "decpos")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    cPanel->slotPosMinus();
    return;
  }
  if (cmd == "quit")
  {
    kdDebug(555) << i18n("Execute remote command") << " '" << cmd << "'...\n";
    slotQuit();
    return;
  }
  
  kdDebug(555) << "No such command\n";
}
  

void KMP::dragEnterEvent (QDragEnterEvent *dev)
{
  // dev->accept(true);
   dev->accept(QUriDrag::canDecode(dev) || QTextDrag::canDecode(dev));
}


void KMP::dropEvent(QDropEvent* dev)
{                                                              

  KURL::List uris;
  
  if (KURLDrag::decode(dev, uris))
  {
    playList->slotClearList();
    playList->Add(uris, NULL);
    slotPlay();
  }
  else 
  if (strcmp(dev->format(), "text/x-moz-url") == 0)    // for mozilla drops
  {
    playList->slotClearList();
    QByteArray data = dev->encodedData("text/plain");
    QString md(data);
    playList->Add(md, NULL);
    slotPlay();
  }  

}


void KMP::showEvent (QShowEvent *)
{
  if ( (videoWin->xineRunning) && (currentVisualPlugin != "none") )
   videoWin->SetVisualPlugin( currentVisualPlugin );
}  


void KMP::hideEvent (QHideEvent *)
{
  if ( (videoWin->xineRunning) && (currentVisualPlugin != "none") )
   videoWin->SetVisualPlugin( "none" );
}  


void KMP::closeEvent(QCloseEvent* cev)
{
 cev->accept();
 slotQuit();
}


void KMP::moveEvent(QMoveEvent*)
{ 
  videoWin->GlobalPosChanged();
}



/****************************************
 *   PLAYBACK CONTROL
 ****************************************/

void KMP::slotPlay()
{
  
  if (playSingleUrl) playSingleUrl = false;
  
  QListViewItem* item = playList->GetCurrent();

  if (!item) return;

  QString playURL = item->text(4);
  if (!subtitleFileToUse.isNull())
    playURL = playURL + "#subtitle:" + subtitleFileToUse;

  if (videoWin->PlayMRL(playURL, item->text(1), true))
  {
    cPanel->SetPlaying();
    fileOpenRecent->addURL(item->text(4));
    if (item->parent())
    {
      cTitle = item->parent()->text(1);
      cUrl = item->parent()->text(4);
    }
    else
    {
      cTitle = item->text(1);
      cUrl = item->text(4);
    }  
    systemTray->SetPlaying(cTitle);
    systemTray->slotPlayIcon();
    fsPanel->slotSetTitle( cTitle );
  }  
  else
   if (cPanel->GetPlaying())
   {
     slotStop();
   }  
}


void KMP::slotPlayNext()
{
  if (hasChapters)
  {
    videoWin->PlayNextChapter();
    return;
  }
    
  if (playSingleUrl) return;
    
  QListViewItem* item = playList->GetNext();

  if (!item) return;

  QString playURL = item->text(4);
  if (!subtitleFileToUse.isNull())
    playURL = playURL + "#subtitle:" + subtitleFileToUse;

  if (videoWin->PlayMRL(playURL, item->text(1), true))
  {
    cPanel->SetPlaying();
    fileOpenRecent->addURL(item->text(4));
    if (item->parent())
    {
      cTitle = item->parent()->text(1);
      cUrl = item->parent()->text(4);
    }
    else
    {
      cTitle = item->text(1);
      cUrl = item->text(4);
    }  
    systemTray->SetPlaying(cTitle);
    systemTray->slotPlayIcon();
    fsPanel->slotSetTitle( cTitle );
  }  
  else 
  {
    slotStop();
  }
}
 


void KMP::slotPlayPrev()
{

  if (hasChapters)
  {
    videoWin->PlayPreviousChapter();
    return;
  }
  
  if (playSingleUrl) return;
  
   QListViewItem* item = playList->GetPrevious();

   if (!item) return;

   QString playURL = item->text(4);
   if (!subtitleFileToUse.isNull())
     playURL = playURL + "#subtitle:" + subtitleFileToUse;
   
   if (videoWin->PlayMRL(playURL, item->text(1), true))
   {
     cPanel->SetPlaying();
     fileOpenRecent->addURL(item->text(4));
     if (item->parent())
     {
       cTitle = item->parent()->text(1);
       cUrl = item->parent()->text(4);
     }
     else
     {
       cTitle = item->text(1);
       cUrl = item->text(4);
     }  
     systemTray->SetPlaying(cTitle);
     systemTray->slotPlayIcon();
     fsPanel->slotSetTitle( cTitle );
   } 
   else 
   {
     slotStop();
   }
}


void KMP::slotEndOfPlayback()
{
  if (!cPanel->GetPlaying()) return; //end of logo, error

  if (playSingleUrl)
  {
    slotStop();
    return;
  }

  
   QListViewItem* item = playList->GetNext();

   if (!item)
   {
     slotStop();
   }
   else
   {  
     QString playURL = item->text(4);
     if (!subtitleFileToUse.isNull())
       playURL = playURL + "#subtitle:" + subtitleFileToUse;

     if (videoWin->PlayMRL(playURL, item->text(1), true))
     {
       fileOpenRecent->addURL(item->text(4));
       if (item->parent())
       {
         cTitle = item->parent()->text(1);
         cUrl = item->parent()->text(4);
       }
       else
       {  
         cTitle = item->text(1);
         cUrl = item->text(4);
       }  
       systemTray->SetPlaying(cTitle);
       systemTray->slotPlayIcon();
       fsPanel->slotSetTitle( cTitle );
     } 
     else 
     {
       slotStop();
     }
   }   
}


void KMP::slotStop()
{
  videoWin->slotStopPlayback();
  cPanel->ResetControlPanel();
  systemTray->slotNormalIcon();
  playSingleUrl = false;
  cTitle = QString::null;
  cUrl = QString::null;
  tRow->slotChangeText(i18n("Ready."));
}


void KMP::slotPlayRecent(const KURL& url)
{

  QString title;
  
  if (url.isLocalFile())
       title = url.fileName();
     else
       title = url.prettyURL();

  //kdDebug(555) << "Play recent url: " << url.url() << endl;

  QString playURL = url.url();
  if (!subtitleFileToUse.isNull())
    playURL = playURL + "#subtitle:" + subtitleFileToUse;
  
  if (videoWin->PlayMRL(playURL, title, false))
  {
    cPanel->SetPlaying();
    playSingleUrl = true;
    cUrl = url.prettyURL();
    cTitle = title;
    systemTray->SetPlaying(cTitle);
    systemTray->slotPlayIcon();
    fsPanel->slotSetTitle( cTitle );
    fsPanel->slotSetStreamInfo( videoWin->GetStreamInfo() );
    fsPanel->slotSetLength( videoWin->GetLengthInfo() );
  }
  else
    if (cPanel->GetPlaying())
    {
      slotStop();
    }  
}


/*************************************************/

void KMP::slotGetNewTitle(const QString& title)
{
  if (!hasChapters)
    osDisplay->slotOsdShow(title);

  systemTray->SetPlaying(title);
  fsPanel->slotSetTitle( title );
  fsPanel->slotSetStreamInfo( videoWin->GetStreamInfo() );
  fsPanel->slotSetLength( videoWin->GetLengthInfo() );
}


void KMP::slotPlayFiles(const KURL::List& urlList)
{
  playList->slotClearList();
  playList->Add(urlList, NULL);
  slotPlay();
}  
  

void KMP::slotQuit()
{
  if (videoWin->xineRunning)  
  {
    videoWin->SaveXineConfig();
    videoWin->slotRemoveAllFilters();
    playList->slotSaveCurrentPlaylist();
    SaveConfig();
  }  
   
  KApplication::exit(0);
}  


/*********************************************/


void KMP::slotOpenFile()
{
 
  KURL::List urlList;

  urlList = KFileDialog::getOpenURLs(QDir::homeDirPath(), QString::null, 0, i18n("Open File(s)"));

  if (!urlList.count()) return;

  playList->slotClearList();
  playList->Add(urlList, NULL);

  QTimer::singleShot(200, this, SLOT(slotPlay()));
}


void KMP::slotOpenUrl()
{
  KURL kurl = KURLRequesterDlg::getURL(QString::null, 0, i18n("Open URL"));

  if (!kurl.isValid()) return;
  
  playList->slotClearList();
  playList->Add(kurl, NULL);

  QTimer::singleShot(200, this, SLOT(slotPlay()));
}


void KMP::slotOpenSubtitleFile()
{
  KURL kurl = KFileDialog::getOpenURL(QDir::homeDirPath(), i18n("*.smi *.srt *.sub *.txt|Subtitle Files\n*.*|All Files"), 0, i18n("Open Subtitle File"));

  if (!kurl.isValid()) return;

  subtitleFile = kurl.path();
  subtitleFileToUse = subtitleFile;
  playUseSubtitleFile->setChecked(true);
  if (cPanel->GetPlaying())
    slotPlay();  /* restart playback with subtitles */
}


void KMP::slotSaveStreamAs()
{
  QListViewItem* item = playList->GetCurrent();
  if (!item) return;

  KURL kurl = KFileDialog::getSaveURL(QDir::homeDirPath() + "/" + KURL( item->text(4) ).fileName(), QString::null, 0, i18n("Save Stream As..."));

  if (!kurl.isValid()) return;

  videoWin->SetStreamSaveDir( kurl.directory() ); /* FIXME */
  
  QString playURL = item->text(4);
  playURL = playURL + "#save:" + kurl.path();

  if (videoWin->PlayMRL(playURL, item->text(1), true))
   {
     cPanel->SetPlaying();
     cPanel->SetRecording();
     fileOpenRecent->addURL(item->text(4));
     if (item->parent())
     {
       cTitle = item->parent()->text(1);
       cUrl = item->parent()->text(4);
     }
     else
     {
       cTitle = item->text(1);
       cUrl = item->text(4);
     }  
     systemTray->SetPlaying(cTitle);
     systemTray->slotRecordIcon();
   }
   else
   {
     slotStop();
   }
}


void KMP::slotUseSubtitleFile()
{
  if (playUseSubtitleFile->isChecked())
  {
    if (subtitleFile.isNull())
    {
      slotOpenSubtitleFile();
      return;
    }  
    subtitleFileToUse = subtitleFile;
    if (cPanel->GetPlaying())
      slotPlay();
  }
  else
  {  
    subtitleFileToUse = QString::null;
    if (cPanel->GetPlaying())
      slotPlay();
  }  
}
              

void KMP::slotToggleFullscreen()
{
  if (viewFullscreen->isChecked())
  {
   /** go fullscreen **/
  
    menuBar->hide();
    tRow->hide();
    cPanel->hide();
  
    showFullScreen();
    videoWin->StartMouseHideTimer();
  }
  else
  {
    
    tRow->show();

    if (!viewMinimalMode->isChecked())
    {
     menuBar->show();
     cPanel->show();
    }
    
    showNormal();
    videoWin->StopMouseHideTimer();
  }
}


void KMP::slotToggleMinimalMode()
{
  if (viewFullscreen->isChecked()) return;

  if (viewMinimalMode->isChecked())
  {
    /** go to minimal mode **/
  
    menuBar->hide();
    cPanel->hide();
  }  
  else
  {
    menuBar->show();
    cPanel->show();
  }  
}
    

void KMP::slotPlayAutoplayPlugin(const QString& plugin)
{
  QStringList mrlList;
  
  if (videoWin->GetAutoplayPluginMrl(plugin, mrlList));
  {
    playList->slotClearList();
    playList->AddAutoplayUrls(mrlList, NULL);
    slotPlay();
  }
}


void KMP::slotAddAutoplayUrls(const QString& plugin, QListViewItem* after)
{
  QStringList mrlList;

  if (videoWin->GetAutoplayPluginMrl(plugin, mrlList));
  {
    playList->AddAutoplayUrls(mrlList, after);
  }
}

    
void KMP::slotHasChapters(bool c)
{
  hasChapters = c;
}  


void KMP::slotShowPlayList()
{
  playList->show();
  playList->raise();
  if (playList->isMinimized())
    playList->showNormal();
  playList->setActiveWindow();
}


void KMP::slotShowSetup()
{
  configWin->show();
}      


void KMP::slotShowFilterConfig()
{
  filterDialog->show();
}  


void KMP::slotEnableFilters()
{
  videoWin->slotEnableFilters( viewPostprocess->isChecked() );
}  


void KMP::slotEndlessMode()
{
  if (playEndlessPlay->isChecked())
    playList->SetEndless(true);
   else
    playList->SetEndless(false);
}


void KMP::slotRandomPlay()
{
  if (playRandomPlay->isChecked())
    playList->SetRandomPlay(true);
   else
    playList->SetRandomPlay(false);
}


void KMP::slotToggleBroadcastMaster()
{
  bool ok = false;

  if (broadcastMaster->isChecked())
  {
    broadcastPort = (uint)QInputDialog::getInteger( QString::null, i18n("Broadcasting Port"), broadcastPort, 0, 1000000, 1, &ok);
    if (!ok)
    {
      broadcastMaster->setChecked( false );
      return;
    }
    videoWin->SetBroadcasterPort( broadcastPort );  
  }
  else
  {
    videoWin->SetBroadcasterPort(0); /* disable */  
  }
}


/************ network broadcasting ***************/

void KMP::slotBroadcastSlave()
{

  KDialogBase* dialog = new KDialogBase( 0, "configmaster", true, QString::null, KDialogBase::Ok|KDialogBase::Cancel );
  QVBox* page = dialog->makeVBoxMainWidget();
  new QLabel(i18n("Master Address"), page);
  KLineEdit* address = new KLineEdit( broadcastMasterAddress, page);
  new QLabel(i18n("Port"), page);
  QSpinBox* port = new QSpinBox(0, 1000000, 1, page);
  port->setValue( broadcastPort );
  
  int button = dialog->exec();

  if (button == KDialogBase::Accepted)
  {
    // kdDebug(555) << "Apply master configuration" << endl;
     broadcastPort = port->value();
     broadcastMasterAddress = address->text();
  }   

  delete dialog;

  if (button == KDialogBase::Rejected) return;
  
  slotPlayRecent( QString("slave://") + broadcastMasterAddress + ":" + QString::number(broadcastPort) );
}              

/*************/


void KMP::slotGenConfig(bool loadPl)
{
  loadPlaylistAtStartup = loadPl;
}


void KMP::slotConfigXine()
{
   new XineConfig(videoWin->GetXineEngine());
}


void KMP::slotVideoSettings()
{
  videoSettings->show();
}  


void KMP::slotEqualizer()
{
  equalizer->show();
}  


void KMP::slotXineMessage(const QString& message)
{
  xineMessage = message;
  QTimer::singleShot(100, this, SLOT( slotShowMessage() ));
}  


void KMP::slotShowMessage()
{
  KMessageBox::sorry(0, xineMessage, i18n("xine Message"));
}  


/*********************************************************
 *   TAKE A SCREENSHOT
 *********************************************************/

void KMP::slotScreenshot()
{
  int width, height;
  double scaleFactor;
  uchar* rgb32BitData = NULL;

  videoWin->GetScreenshot(rgb32BitData, width, height, scaleFactor);

  if (rgb32BitData == NULL)
  {
    kdDebug(555) << "Can't take screenshot\n";
    return;
  }

  QImage shot(rgb32BitData, width, height, 32 , (uint*)&RGB_MASK , 0  , QImage::IgnoreEndian);

 
  if (scaleFactor >= 1.0)
    width = (int)((double) width * scaleFactor);  
  else
    height = (int) ((double) height / scaleFactor);
    
  shot = shot.smoothScale(width, height);
  

/** save **/


  KFileDialog dlg(QDir::homeDirPath(), i18n("*.png|PNG-File\n*.bmp|BMP-File\n*.xbm|XBM-File"),
                          0, "save screenshot", true);

  dlg.setOperationMode(KFileDialog::Saving);
  dlg.setCaption(i18n("Save screenshot as..."));
  dlg.setSelection("screenshot.png");

  ScreenshotPreview* prev = new ScreenshotPreview(shot, &dlg);
  dlg.setPreviewWidget(prev);

  dlg.exec();

  QString fileName = dlg.selectedFile();

  if (fileName.isEmpty())
  {
    delete [] rgb32BitData;
    return;
  }  

  QString type = dlg.currentFilter();
  type = (type.remove(0,2)).upper();
  kdDebug(555) << "Save screenshot as " << type << "\n";
  
 
  if  (!shot.save(fileName, type))
    kdError(555) << i18n("Screenshot not saved successfully!") << "\n";
    

  delete [] rgb32BitData;
}


/*********************************/


void KMP::slotToggleTv()
{
   /* not implemented yet */
}



/******* kbookmarkowner reimplementation ************/


void KMP::openBookmarkURL(const QString& url)
{
  playList->slotClearList();
  playList->Add(url, NULL);   // allows playlists as bookmark
  slotPlay();

  //slotPlayRecent(url);
}


QString KMP::currentTitle() const
{
   return cTitle;
}


QString KMP::currentURL() const
{
  return cUrl;
}

