/***************************************************************************
                           kaffeine_part.cpp - kpart for embedded konqueror preview
                             -------------------
    begin                : Don Apr 17 11:30:44 CEST 2003
    revision             : $Revision: 1.24 $
    last modified        : $Date: 2004/01/11 22:24:48 $ by $Author: juergenk $
    copyright            : (C) 2003-2004 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <kiconloader.h>
#include <klocale.h>
#include <kaction.h>
#include <kglobalsettings.h>
#include <kdebug.h>
#include <kapplication.h>
#include <kparts/genericfactory.h>

#include <dcopclient.h>

#include <qtimer.h>
#include <qgrid.h>
#include <qtooltip.h>


#include "kaffeine_part.h"
#include "kaffeine_part.moc"


typedef KParts::GenericFactory<KaffeinePart> KaffeinePartFactory;
K_EXPORT_COMPONENT_FACTORY (libkaffeinepart, KaffeinePartFactory);



KaffeinePart::KaffeinePart(QWidget* parentWidget, const char* widgetName, QObject* parent, const char* name, const QStringList&)
: KParts::ReadOnlyPart(parent, name ? name : "KaffeinePart"), videoWin(NULL), infoRow(NULL), playList(NULL), stopped(true)
{
  kdDebug(555) << "KaffeinePart: Creating new..." << endl;

  KInstance* instance = KaffeinePartFactory::instance();
  setInstance(instance);

  QWidget* mainFrame = new QWidget(parentWidget, widgetName);
  QVBoxLayout* boxLayout = new QVBoxLayout(mainFrame);
  QWidget* panel = new QWidget(mainFrame, "panel");
  panel->setMaximumHeight(32);
  QHBoxLayout* panelLayout = new QHBoxLayout(panel);
  panelLayout->setSpacing(5);
  panelLayout->setMargin(2);
  
  videoWin = new VideoWindow(mainFrame, "videowin", QString::null, QString::null, true);
  connect(videoWin, SIGNAL(signalPlaybackFinished()), this, SLOT(slotPlaybackFinished()));
  connect(videoWin, SIGNAL(signalNewTitle(const QString&)), this, SLOT(slotNewTitle(const QString&)));

  infoRow = new TextRow(mainFrame, "inforow");
  connect(videoWin, SIGNAL(signalNewInfo(const QString&)), infoRow, SLOT(slotChangeText(const QString&)));

  KPushButton* previousButton = new KPushButton(panel);
  previousButton->setPixmap(KGlobal::iconLoader()->loadIcon("player_start", KIcon::Small));
  connect(previousButton, SIGNAL(clicked()), this, SLOT(slotPrevious()));

  KPushButton* playButton = new KPushButton(panel);
  playButton->setPixmap(KGlobal::iconLoader()->loadIcon("player_play", KIcon::Small));
  connect(playButton, SIGNAL(clicked()), this, SLOT(slotPlay()));
  
  KPushButton* stopButton = new KPushButton(panel);
  stopButton->setPixmap(KGlobal::iconLoader()->loadIcon("player_stop", KIcon::Small));
  connect(stopButton, SIGNAL(clicked()), this, SLOT(slotStop()));
  
  KPushButton* nextButton = new KPushButton(panel);
  nextButton->setPixmap(KGlobal::iconLoader()->loadIcon("player_end", KIcon::Small));
  connect(nextButton, SIGNAL(clicked()), this, SLOT(slotNext()));

  posSlider = new QSlider(Qt::Horizontal, panel);
  posSlider->setRange(0,65535);
  posSlider->setSteps(100, 1000);
  connect(posSlider, SIGNAL(sliderMoved(int)), videoWin, SLOT(slotChangePosition(int)));
  
  timeButton = new KPushButton(panel);
  connect(timeButton, SIGNAL(clicked()), videoWin, SLOT(slotTogglePlayMode()));
  connect(videoWin, SIGNAL(signalNewPosition(int, const QString&)), this, SLOT(slotSetPosition(int, const QString&)));

  KPushButton* logoButton = new KPushButton(panel);
  logoButton->setPixmap(KGlobal::iconLoader()->loadIcon("kaffeine", KIcon::Small));
  connect(logoButton, SIGNAL(clicked()), this, SLOT(slotSendURL()));

  QToolTip::add(timeButton,i18n("Change Counter-Mode"));
  QToolTip::add(nextButton, i18n("Next Entry/Chapter"));
  QToolTip::add(previousButton, i18n("Previous Entry/Chapter"));
  QToolTip::add(stopButton, i18n("Stop"));
  QToolTip::add(logoButton, i18n("Send URL to Kaffeine (append to playlist)"));
  
  panelLayout->addWidget(playButton);
  panelLayout->addWidget(stopButton);
  panelLayout->addWidget(previousButton);
  panelLayout->addWidget(nextButton);
  panelLayout->addWidget(posSlider);
  panelLayout->addWidget(timeButton);
  panelLayout->addWidget(logoButton);
  
  boxLayout->addWidget(videoWin);
  boxLayout->addWidget(infoRow);
  boxLayout->addWidget(panel);


/*** load configuration ***/

  KConfig* config = instance->config();
  config->setGroup("General Options");
  
  QString metaStr = config->readEntry("Meta String", QString("%1 - %2 (%3)"));
  videoWin->slotSetConfig( metaStr, 0 );

  QFont f = KGlobalSettings::generalFont();
  QFont infoFont = config->readFontEntry("Textrow Font", &f);
  QColor c = KGlobalSettings::baseColor();
  QColor infoColor = config->readColorEntry("Textrow Color", &c);
  infoRow->slotSetConfig( &infoFont, &infoColor );

  config->setGroup("Visualization");
  visualPlugin = config->readEntry("Visual Plugin", "goom");
  
/*********/

  mainFrame->show();
  setWidget(mainFrame);

  oldPosition = videoWin->mapToGlobal( QPoint(0,0) );
  connect( &posCheckTimer, SIGNAL( timeout() ), this, SLOT( slotCheckMoved() ) );
  posCheckTimer.start( 333 );
}


KaffeinePart::~KaffeinePart()
{
 kdDebug(555) << "KaffeinePart destructor..." << endl;
 posCheckTimer.stop();
 delete playList;
 playList = NULL;
}


KAboutData *KaffeinePart::createAboutData()
{
    KAboutData* aboutData = new KAboutData( "kaffeine", "Kaffeine",
    VERSION, I18N_NOOP("A xine based media player. Take a look at Kaffeine Handbook for more information."),
    KAboutData::License_GPL,
    "(c) 2003, Jürgen Kofler.", 0, "http://kaffeine.sourceforge.net", "kaffeine@gmx.net");
    aboutData->addAuthor("Jürgen Kofler.",0, "kaffeine@gmx.net");

   return aboutData;
}


bool KaffeinePart::openURL(const KURL& url)
{
  kdDebug(555) << "KaffeinePart::openURL()" << endl;
  if (!url.isValid()) return false;
  m_url = url;
  m_bTemp = false;

  if (!videoWin->xineRunning)
  {
   /* start xine manual, because polish() sometimes called too early */
    if (!videoWin->InitXine())
    {
      emit canceled(i18n("Can't init Xine Engine!"));
      return false;
    }
    videoWin->SetVisualPlugin( visualPlugin );
    playList = new PlayList(0, "playlist", false /* without GUI */);
    connect(videoWin, SIGNAL(signalNewMrlReference(const QString&)), playList, SLOT(slotAddMrl(const QString&)));
    connect(videoWin, SIGNAL(signalMetaInfo(const QString&)), playList, SLOT(slotGetMetaInfo(const QString&)));
    kdDebug(555) << "KaffeinePart: Got URL: " << m_url.url() << endl;
    playList->Add(m_url, 0);

    if (!playList->GetCurrent())
    {
      kdWarning(555) << "KaffeinePart: No valid url..." << endl;    
      return false;
    }  
  } 

  QTimer::singleShot(100, this, SLOT(slotPlay()));

  return true;
}


bool KaffeinePart::closeURL()
{
  kdDebug(555) << "KaffeinePart::closeURL()" << endl;
  if (playList)
   playList->slotClearList();
  return true;
}   
  

void KaffeinePart::slotPlay()
{
  kdDebug(555) << "KaffeinePart::slotPlay()" << endl;

  if (!videoWin->xineRunning) return;  /** failed to start xine-engine **/

  stopped = false;
  
  QListViewItem* item = playList->GetCurrent();

  if (!item) return;

  if (!videoWin->PlayMRL(item->text(4), item->text(1), true))
    Reset();
   else
     emit setWindowCaption( item->text(1) );
}


void KaffeinePart::slotStop()
{
  kdDebug(555) << "KaffeinePart::slotStop()" << endl;

  if (!videoWin->xineRunning) return;  /** failed to start xine-engine **/
  
  Reset();
  infoRow->slotChangeText(i18n("Ready."));  
}


void KaffeinePart::slotNext()
{
  if (stopped) return;

  QListViewItem* item = playList->GetNext();

  if (!item) return;

  if (!videoWin->PlayMRL(item->text(4), item->text(1), true))
    Reset();
   else
    emit setWindowCaption( item->text(1) ); 
}


void KaffeinePart::slotPrevious()
{
  if (stopped) return;

  QListViewItem* item = playList->GetPrevious();

  if (!item) return;

  if (!videoWin->PlayMRL(item->text(4), item->text(1), true))
    Reset();
   else
    emit setWindowCaption( item->text(1) );  
}


void KaffeinePart::slotPlaybackFinished()
{
  if (stopped) return;  /* end of logo */

  QListViewItem* item = playList->GetNext();

  if (!item)
  {
    slotStop();
    return;
  }  
   
  if (!videoWin->PlayMRL(item->text(4), item->text(1), true))
     Reset();
    else
      emit setWindowCaption( item->text(1) );
}


void KaffeinePart::slotNewTitle(const QString& title)
{
  emit setWindowCaption( title );
}  


void KaffeinePart::Reset()
{
  stopped = true;

  videoWin->slotStopPlayback();
  emit setWindowCaption("Kaffeine Media Player " + QString(VERSION));
  slotSetPosition(0, "0:00:00");
}

  
void KaffeinePart::slotSendURL()
{
  DCOPClient* client = KApplication::dcopClient();
  
  if (!client->send( "kaffeine", "Kaffeine", "appendURL(QString)", m_url.prettyURL() ))
  {
    kdDebug(555) << "DCOP: Unable to send the URL to Kaffeine (not running?)" << endl;
  }  
}


void KaffeinePart::slotSetPosition(int pos, const QString& time)
{
  posSlider->setValue(pos);
  timeButton->setText(time);
}  


/* check if konqueror was moved, send new global position
   of the video window to xine */
void KaffeinePart::slotCheckMoved()
{
  if (!stopped) return;

  QPoint newPos = videoWin->mapToGlobal( QPoint(0,0) );
  if ( newPos != oldPosition )
  {
     videoWin->GlobalPosChanged();
     oldPosition = newPos;
  }   
}  




