/***************************************************************************
                          postfilter.cpp - wrapper for xine's postprocessing filters
                             -------------------
    begin                : Son Dez 7 2003
    revision             : $Revision: 1.6 $
    last modified        : $Date: 2004/01/11 13:07:32 $ by $Author: juergenk $
    copyright            : (C) 2003-2004 by Juergen  Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kcombobox.h>
#include <knuminput.h>
#include <kpushbutton.h>
#include <kseparator.h>
#include <klineedit.h>
#include <klocale.h>
#include <kdebug.h>

#include <qlayout.h>
#include <qcheckbox.h>
#include <qlabel.h>

#include <stdio.h>
 
#include "postfilter.h"
#include "postfilter.moc"



PostFilter::PostFilter(const QString& name, xine_t* engine, xine_audio_port_t* audioDriver,
              xine_video_port_t* videoDriver, QWidget *parent) : QObject(parent), data(NULL), groupBox(NULL)
{
  filterName = name;
  kdDebug(555) << "Create Postprocessing Filter: " << filterName << endl;

  xine_post_in_t* inputAPI = NULL;

  groupBox = new QGroupBox( name, parent );
  groupBox->setSizePolicy ( QSizePolicy (QSizePolicy::Minimum, QSizePolicy::Fixed) );
  QGridLayout* grid = new QGridLayout( groupBox, 2, 2 );
  grid->setMargin( 20 );
  grid->setSpacing( 5 );
  int row = 0;

  xineEngine = engine;

  xinePost = xine_post_init( xineEngine, filterName, 0, &audioDriver, &videoDriver );
  if ((inputAPI = (xine_post_in_t*)xine_post_input( xinePost, const_cast<char*>("parameters")) ))
  {
    xinePostAPI = (xine_post_api_t*)inputAPI->data;
    xinePostDescr = xinePostAPI->get_param_descr();
    xinePostParameter = xinePostDescr->parameter;

    data = new char[ xinePostDescr->struct_size ];
    xinePostAPI->get_parameters( xinePost, data );

    QLabel* descr;
    
    while (xinePostParameter->type != POST_PARAM_TYPE_LAST)
    {
      kdDebug(555) << "Parameter: " << xinePostParameter->name << endl;
      if (xinePostParameter->readonly) continue;

      if (xinePostParameter->enum_values)
      {
        char** enums = xinePostParameter->enum_values;
        KComboBox* comboBox = new KComboBox( groupBox );
        for (int i = 0; enums[i]; i++)
        {
          comboBox->insertItem(enums[i]);
        }  
        comboBox->setCurrentItem( *(int*)(data+xinePostParameter->offset) );
        PostFilterHelper* helper = new PostFilterHelper( xinePostParameter->offset, comboBox );
        connect( comboBox, SIGNAL( activated( int )), helper, SLOT( slotIntValue( int ) ));
        connect( helper, SIGNAL( signalIntValue(int, int)), this, SLOT( slotApplyIntValue(int, int) ));

        grid->addWidget( comboBox, row, 0 );
        descr = new QLabel(QString::fromLocal8Bit( xinePostParameter->description ), groupBox );
        descr->setAlignment( QLabel::WordBreak | QLabel::AlignVCenter );
        grid->addWidget( descr, row, 1 );
        row++;
        xinePostParameter++;
        continue;
      }

      switch (xinePostParameter->type)
      {
        case POST_PARAM_TYPE_INT:
        {
          KIntNumInput* numInput = new KIntNumInput( *(int*)(data+xinePostParameter->offset), groupBox );
          numInput->setRange( (int)xinePostParameter->range_min, (int)xinePostParameter->range_max, 1, false);
          PostFilterHelper* helper = new PostFilterHelper( xinePostParameter->offset, numInput );
          connect( numInput, SIGNAL( valueChanged( int ) ), helper, SLOT( slotIntValue( int ) ));
          connect( helper, SIGNAL( signalIntValue(int, int) ), this, SLOT( slotApplyIntValue(int, int) ));    

          grid->addWidget( numInput, row, 0 );
          break;
        }
        case POST_PARAM_TYPE_DOUBLE:
        {
          KDoubleNumInput* numInput = new KDoubleNumInput( *(double*)(data+xinePostParameter->offset), groupBox );
          numInput->setRange( xinePostParameter->range_min, xinePostParameter->range_max, 0.01, false);
          PostFilterHelper* helper = new PostFilterHelper( xinePostParameter->offset, numInput );
          connect( numInput, SIGNAL( valueChanged( double ) ), helper, SLOT( slotDoubleValue( double ) ));
          connect( helper, SIGNAL( signalDoubleValue(int, double) ), this, SLOT( slotApplyDoubleValue(int, double) ));

          grid->addWidget( numInput, row, 0 );
          break;  
        }
        case POST_PARAM_TYPE_CHAR:
        {
          KLineEdit* charInput = new KLineEdit( (char*)(data+xinePostParameter->offset), groupBox );
          charInput->setMaxLength(256);
          size = xinePostParameter->size;
          PostFilterHelper* helper = new PostFilterHelper( xinePostParameter->offset, charInput );
          connect( charInput, SIGNAL( returnPressed( const QString& ) ), helper, SLOT( slotCharValue( const QString& ) ));
          connect( helper, SIGNAL( signalCharValue(int, const QString&) ), this, SLOT( slotApplyCharValue(int, const QString& ) ));

          grid->addWidget( charInput, row, 0 );
          break;
        }  
        case POST_PARAM_TYPE_STRING:
        case POST_PARAM_TYPE_STRINGLIST: break; /* not implemented */
        case POST_PARAM_TYPE_BOOL:
        {
          QCheckBox* checkBox = new QCheckBox( groupBox );
          checkBox->setChecked( bool(*(int*)(data+xinePostParameter->offset)) );
          PostFilterHelper* helper = new PostFilterHelper( xinePostParameter->offset, checkBox );
          connect( checkBox, SIGNAL( toggled( bool ) ), helper, SLOT( slotBoolValue( bool ) ));
          connect( helper, SIGNAL( signalIntValue(int, int) ), this, SLOT( slotApplyIntValue(int, int) ));

          grid->addWidget( checkBox, row, 0 );
          break;
        }
        default: break;
      }

      descr = new QLabel( QString::fromLocal8Bit( xinePostParameter->description ), groupBox );
      descr->setAlignment( QLabel::WordBreak | QLabel::AlignVCenter );
      grid->addWidget( descr, row, 1 );
      row++;
      xinePostParameter++;
    }
  }         
  KSeparator* sep = new KSeparator( KSeparator::Horizontal, groupBox );
  grid->addMultiCellWidget( sep, row, row, 0, 1 );
  row++;
  KPushButton* deleteButton = new KPushButton( i18n("Delete Filter"), groupBox );
  deleteButton->setSizePolicy ( QSizePolicy (QSizePolicy::Minimum, QSizePolicy::Fixed) );
  connect( deleteButton, SIGNAL( clicked() ), this, SLOT( slotDeletePressed() ));
  grid->addWidget( deleteButton, row, 0 );
  groupBox->show();
}



PostFilter::~PostFilter()
{
  kdDebug(555) << "Delete Postprocessing Filter: " << filterName << endl;
  delete groupBox;
  delete data;
  xine_post_dispose(xineEngine, xinePost);
}


void PostFilter::slotApplyIntValue(int offset, int val)
{
  kdDebug(555) << "PostFilter: " << filterName << " Apply integer value " << val << " on offset " << offset << endl;
  *(int*)(data+offset) = val;
  xinePostAPI->set_parameters( xinePost, data );
}


void PostFilter::slotApplyDoubleValue(int offset, double val)
{
  kdDebug(555) << "PostFilter: " << filterName << " Apply double value " << val << " on offset " << offset << endl;
  *(double*)(data+offset) = val;
  xinePostAPI->set_parameters( xinePost, data );
}


void PostFilter::slotApplyCharValue(int offset, const QString& val)
{
  kdDebug(555) << "PostFilter: " << filterName << " Apply char value '" << val << "' on offset " << offset << endl;
  snprintf((char *)(data+offset), size / sizeof(char), "%s", val.latin1());  /* copy with defined max length */
  xinePostAPI->set_parameters( xinePost, data );
}


xine_post_in_t* PostFilter::GetInput() const
{
  kdDebug(555) << "PostFilter: Get input 'video'" << endl;
  return (xine_post_in_t*)xine_post_input( xinePost, const_cast<char*>("video") );
}
  

xine_post_out_t* PostFilter::GetOutput() const
{
  const char *const *outs = xine_post_list_outputs( xinePost );
  kdDebug(555) << "PostFilter: Get ouput '" << (char*)*outs << "'" << endl;
  return (xine_post_out_t*)xine_post_output( xinePost, (char *) *outs);
}




  





