/***************************************************************************
                           postfilter.h - wrapper for xine's postprocessing filters
                             -------------------
    begin                : Son Dez 7 2003
    revision             : $Revision: 1.3 $
    last modified        : $Date: 2004/01/11 12:35:40 $ by $Author: juergenk $
    copyright            : (C) 2003-2004 by Juergen  Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef POSTFILTER_H
#define POSTFILTER_H

#include <qwidget.h>
#include <qobject.h>
#include <qstring.h>
#include <qgroupbox.h>

#include <xine.h>

/**
  *@author Juergen  Kofler
  */


class PostFilterHelper : public QObject
{
  Q_OBJECT
public:
  PostFilterHelper( int _offset, QWidget* parent ) : QObject( parent) { offset = _offset; }
  ~PostFilterHelper() {};
  
signals:
  void signalIntValue( int, int );
  void signalDoubleValue( int, double );
  void signalCharValue( int, const QString&);

public slots:
  void slotIntValue( int val ) { emit signalIntValue( offset, val ); }
  void slotDoubleValue( double val ) { emit signalDoubleValue( offset, val ); }
  void slotBoolValue( bool val ) { emit signalIntValue( offset, (int)val ); }  
  void slotCharValue( const QString& val ) { emit signalCharValue( offset, val ); }
  
private:
  int offset;
  
};


  
class PostFilter : public QObject
{
   Q_OBJECT
public: 
   PostFilter(const QString& name, xine_t* engine, xine_audio_port_t* audioDriver,
              xine_video_port_t* videoDriver, QWidget *parent);
  ~PostFilter();

  xine_post_in_t* GetInput() const;
  xine_post_out_t* GetOutput() const;
  
signals:
  void signalDeleteMe( PostFilter* me );


private slots:
  void slotDeletePressed() { emit signalDeleteMe( this ); }
  void slotApplyIntValue(int offset, int val); 
  void slotApplyDoubleValue(int offset, double val);
  void slotApplyCharValue(int offset, const QString& val);

private:
  xine_t* xineEngine;
  xine_post_t* xinePost;
  xine_post_api_t* xinePostAPI;
  xine_post_api_descr_t* xinePostDescr;
  xine_post_api_parameter_t* xinePostParameter;
  char* data;
  int size;

  QGroupBox* groupBox;
  QString filterName;  /* for debug */

};  



#endif




