/***************************************************************************
                           urllistview.cpp
                             -------------------
    begin                : Mon Apr 28 2003
    revision             : $Revision: 1.9 $
    last modified        : $Date: 2004/01/11 13:33:29 $ by $Author: juergenk $
    copyright            : (C) 2003-2004 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 
#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kglobalsettings.h>

#include <qfontmetrics.h>

#include "urllistview.h"
#include "urllistview.moc"


UrlListView::UrlListView(QWidget *parent, const char *name ) : KListView(parent,name), listCleared(true), itemOfContextMenu(0)
{
  contextMenu = new KPopupMenu(this);
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("player_play", KIcon::Small), i18n("Play"), this, SLOT(slotPlayItem()));
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("edit", KIcon::Small), i18n("&Edit title"), this, SLOT(slotEditTitle()));
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("editcut", KIcon::Small), i18n("C&ut"), this, SIGNAL(signalCut()));
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("editcopy", KIcon::Small), i18n("&Copy"), this, SIGNAL(signalCopy()));
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("editpaste", KIcon::Small), i18n("&Paste"), this, SIGNAL(signalPaste()));
  contextMenu->insertSeparator();
  contextMenu->insertItem(KGlobal::iconLoader()->loadIconSet("info", KIcon::Small), i18n("&Info"), this, SLOT(slotShowInfo()));

/* width of the "length"-column */
  QFontMetrics met( KGlobalSettings::generalFont() );
  int w1 = met.width(i18n("Length"));
  int w2 =  met.width("5:55:55");

  column2Width = w1 > w2 ? w1 : w2;
  column2Width += 8;
   
  connect(this, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
          this, SLOT(slotShowContextMenu(QListViewItem*, const QPoint&, int)));
}


UrlListView::~UrlListView()
{
}


bool UrlListView::acceptDrag(QDropEvent* event) const
{
 return QUriDrag::canDecode(event) || QTextDrag::canDecode(event) || KListView::acceptDrag(event);
}



QDragObject* UrlListView::dragObject()
{
 // get selected items 
  QPtrList<QListViewItem> selected;
  KURL::List kurlList;
  
  selected = selectedItems();
  for (uint i = 0; i<selected.count(); i++)
  {
    kurlList.append( selected.at(i)->text(4) );
  }  

  return KURLDrag::newDrag( kurlList, this, "kaffeine" );
}


void UrlListView::resizeEvent(QResizeEvent* rev)
{

  setColumnWidth(0, 18);
  setColumnWidth(2, column2Width);  /* width of "lenght" column */
  setColumnWidth(3, 18);
  setColumnWidth(4, 18);
  setColumnWidth(1, contentsRect().width()-column2Width-70);

  KListView::resizeEvent(rev);
}


void UrlListView::clear()
{
  listCleared = true;
 // kdDebug(555) << "Playlist cleared\n";
  
  KListView::clear();
}


void UrlListView::SetCleared(bool cl)
{
  listCleared = cl;
}


bool UrlListView::GetCleared() const   /* was playlist cleared ? */
{
  return listCleared;
}



/********** context menu **********/  

void UrlListView::slotShowContextMenu(QListViewItem* item, const QPoint& pos, int)
{
  if (!item) return;  /* dont allow context menu on empty space */
  
  itemOfContextMenu = item;
  contextMenu->popup(pos);
}


void UrlListView::slotPlayItem()
{
  emit signalPlayItem(itemOfContextMenu);
}
  

void UrlListView::slotEditTitle()
{
  itemOfContextMenu->setRenameEnabled(1, true);
  itemOfContextMenu->startRename(1);
  itemOfContextMenu->setRenameEnabled(1, false);
}
  

void UrlListView::slotShowInfo()
{
  QString num;
  num = num.setNum(childCount());
  QString info = itemOfContextMenu->text(1);
  info.append("\n");
  info = info + i18n("URL") + ": " + itemOfContextMenu->text(4) + "\n";
  info = info + i18n("Length") + ": " + itemOfContextMenu->text(2) + "\n";
  info = info + i18n("Stream") + ": " + itemOfContextMenu->text(3) + "\n";
  info = info + i18n("TOTAL: %1 entries").arg(num);
  
  KMessageBox::information(this, info);
}  


                
  

          

