/***************************************************************************
                           videowindow.cpp 
                           -------------------
    begin                : Fre Apr 18 2003
    revision             : $Revision: 1.39 $
    last modified        : $Date: 2004/01/15 20:12:58 $ by $Author: juergenk $
    copyright            : (C) 2003-2004 by J. Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


 
#include <klocale.h>
#include <kurl.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kmessagebox.h>

#include <qapplication.h>
#include <qdir.h>
#include <qcursor.h>
#include <qimage.h>
#include <qmutex.h>

#include <string.h>
#include <stdlib.h>
#include <xine/xineutils.h>

#include "videowindow.h"
#include "videowindow.moc"

#include <X11/keysym.h>
#ifdef HAVE_XTEST
#include <X11/extensions/XTest.h>
#endif



QMutex mutex( true );  /**** mutex ****/


VideoWindow::VideoWindow(QWidget *parent, const char *name, const QString& prefAudio, const QString& prefVideo, bool startManual)
             : QWidget(parent,name)
{
  xineEngine = NULL; xineStream = NULL; audioDriver = NULL; videoDriver = NULL;
  eventQueue = NULL; xinePost = NULL;  xineRunning = false;
  xineDisplay = NULL; xComWindow = 0; eventThread = 0; currentZoom = 100;
  startXineManual = startManual; filtersEnabled = true; visualPluginName = QString::null;

 
  
  preferedAudio = prefAudio;
  preferedVideo = prefVideo;

  setPaletteBackgroundColor(QColor(0,0,0));
  setMouseTracking(true);  /* get mouse move events */

  xineErrors << i18n("No Error");
  xineErrors << i18n("Xine-Error: No Input Plugin");
  xineErrors << i18n("Xine-Error: No Demux Plugin");
  xineErrors << i18n("Xine-Error: Demux failed");
  xineErrors << i18n("Xine-Error: Malformed MRL");
  xineErrors << i18n("Xine-Error: Unknown error");

  playMode = NORMAL_PLAY;

  filterList.setAutoDelete( true ); /*** delete post plugin on removing from list ***/

  connect(&posTimer, SIGNAL(timeout()), this, SLOT(slotGetPosition()));
  connect(&lengthInfoTimer, SIGNAL(timeout()), this, SLOT(slotEmitLengthInfo()));
  connect(&screensaverTimer, SIGNAL(timeout()), this, SLOT(slotFakeKeyEvent()));
  connect(&mouseHideTimer, SIGNAL(timeout()), this, SLOT(slotHideMouse()));

  setUpdatesEnabled(false); //prevent qt overwrites videowin black
}


VideoWindow::~VideoWindow()
{
  /* "careful" shutdown, maybe xine initialization was not successful */

  xineRunning = false;

  /* stop all timers */
  posTimer.stop();
  screensaverTimer.stop();
  mouseHideTimer.stop();

  if (xineStream)
  {
    mutex.lock();
    xine_close(xineStream);
    mutex.unlock();
  }
    
  if (eventThread)
  {
    XClientMessageEvent cm;
    cm.type = ClientMessage;
    cm.window = xineWindow;
    cm.message_type = atomQuit;
    cm.format = 32;

    XSendEvent(xineDisplay, xComWindow, True, 0, (XEvent*) &cm );  /* send quit signal to event loop */
    XFlush(xineDisplay);

    kdDebug(555) << "Wait for event loop to die...\n";
    pthread_join(eventThread, NULL);
  } 

  
  kdDebug(555) << "Shut down xine engine\n";
  if (xineDisplay)
     LockXineDisplay();

  if (eventQueue)
     xine_event_dispose_queue(eventQueue);
  if (xinePost)
     xine_post_dispose (xineEngine, xinePost);  
  if (xineStream)   
     xine_dispose(xineStream);
  if (audioDriver)   
     xine_close_audio_driver(xineEngine, audioDriver);
  if (videoDriver)   
     xine_close_video_driver(xineEngine, videoDriver);
  if (xineEngine)
     xine_exit(xineEngine);
  xineEngine = NULL;

  if (xineDisplay)
     UnlockXineDisplay();

//  XSync(xineDisplay, false);
  if (xComWindow)
     XDestroyWindow(xineDisplay, xComWindow); /* close communication window */
  if (xineDisplay)   
     XCloseDisplay(xineDisplay);  /* close xine display */
  xineDisplay = NULL;
  kdDebug(555) << "xine closed\n";
}


void VideoWindow::SaveXineConfig()
{
  kdDebug(555) << "Save " << configFile << "\n";
  xine_config_save(xineEngine, configFile);
}  


void VideoWindow::LockXineDisplay() const
{
  XLockDisplay(xineDisplay);
}


void VideoWindow::UnlockXineDisplay() const
{
  XUnlockDisplay(xineDisplay);
}    



/***************************************************
 *         CALLBACKS
 ***************************************************/

void VideoWindow::DestSizeCallback(void* p, int /*video_width*/, int /*video_height*/, double /*video_aspect*/,
                       int* dest_width, int* dest_height, double* dest_aspect)

{
  if (p == NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

  *dest_width = vw->width();
  *dest_height = vw->height();
  *dest_aspect = vw->displayRatio;
}



void VideoWindow::FrameOutputCallback(void* p, int /*video_width*/, int /*video_height*/, double /*video_aspect*/,
                          int* dest_x, int* dest_y, int* dest_width, int* dest_height,
                          double* dest_aspect, int* win_x, int* win_y)

{
  if (p == NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

  *dest_x = 0;
  *dest_y = 0 ;
  *dest_width = vw->width();
  *dest_height = vw->height();
  *win_x = vw->globX;
  *win_y = vw->globY;
  *dest_aspect = vw->displayRatio;
}


void VideoWindow::XineEventListener(void *p, const xine_event_t* xineEvent)
{

  if (p == NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

 switch (xineEvent->type)
  {
    case XINE_EVENT_UI_PLAYBACK_FINISHED:
       {
         vw->signalPlaybackFinished();
         break; 
       }
    case XINE_EVENT_UI_CHANNELS_CHANGED:  /* new channel informations */
      {
        int i,j,channels;
        char* lang = new char[128];     /* more than enough */
        QString num;

        vw->audioCh.clear();
        vw->subCh.clear();

        /*** get audio channels  ***/
        mutex.lock();
        channels = QMAX(6, xine_get_stream_info(vw->xineStream, XINE_STREAM_INFO_MAX_AUDIO_CHANNEL));
        for(i = 0; i < channels; i++)
        {
          if (xine_get_audio_lang(vw->xineStream, i, lang))
            vw->audioCh << lang;
           else
            vw->audioCh << i18n("Ch ") + num.setNum(i+1);
        }     
      
        /*** get subtitle channels ***/
    
        channels = QMAX(6, xine_get_stream_info(vw->xineStream, XINE_STREAM_INFO_MAX_SPU_CHANNEL));
        for(j = 0; j < channels; j++)
        {
          if (xine_get_spu_lang(vw->xineStream, j, lang))
            vw->subCh << lang;
           else
            vw->subCh << i18n("Ch ") + num.setNum(j+1);
        }    

        delete lang;  /* or not? */

        int currentAudio = xine_get_param(vw->xineStream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
        int currentSub = xine_get_param(vw->xineStream, XINE_PARAM_SPU_CHANNEL);
        mutex.unlock();
 
        if (currentAudio > (i-1))
        {
          vw->slotSetAudioChannel(0);
          currentAudio = -1;
        } 
          
        if (currentSub > (j-1))
        {
          vw->slotSetSubtitleChannel(0);
          currentSub = -1;
         }

         vw->signalNewChannels(vw->audioCh, vw->subCh, currentAudio, currentSub);

         break;
      }
    case XINE_EVENT_UI_SET_TITLE:  /* set new title */
       {
        xine_ui_data_t* xd = (xine_ui_data_t*)xineEvent->data;
        QString title = QString::fromLocal8Bit( (char*)xd->str );
        vw->signalNewTitle( title );
        vw->trackInfoRow = title + " ** " + vw->GetStreamInfo() + vw->extraInfo;
        vw->signalNewInfo( vw->trackInfoRow );
        break;
       }
     case XINE_EVENT_PROGRESS:
       {
        xine_progress_data_t* pd = (xine_progress_data_t*)xineEvent->data;

        QString perc;
        perc.setNum(pd->percent);
        perc.append(" %");
        
        QString progress = pd->description;
        progress.append("  ");
        progress.append(perc);
        
        vw->signalNewInfo(progress);

        if (pd->percent == 100)
         vw->signalNewInfo(vw->trackInfoRow);

        break;
        }
     case XINE_EVENT_DROPPED_FRAMES:
      {
        xine_dropped_frames_t* dropped = (xine_dropped_frames_t*)xineEvent->data;

        QString message = i18n("General Warning: \n");
        message = message + i18n("Performance problems detected!") + " ";
        message = message + i18n("(skipped frames: %1% - discarded frames: %2%)").arg( dropped->skipped_frames/10 ).arg( dropped->discarded_frames/10 ) + "\n";
        message = message + i18n("Run 'xine-check' and read the xine FAQ on http://www.xinehq.de");
        vw->signalXineMessage( message );
 
        break;
       }  
     case XINE_EVENT_SPU_BUTTON:
       {
         xine_spu_button_t* button = (xine_spu_button_t*)xineEvent->data;

         if (button->direction == 1) /* enter a button */
           vw->setCursor(QCursor(Qt::PointingHandCursor));
          else
           vw->setCursor(QCursor(Qt::ArrowCursor));

         break;
       }  
     case XINE_EVENT_MRL_REFERENCE:
       {
        xine_mrl_reference_data_t* mrldata = (xine_mrl_reference_data_t*)xineEvent->data;
        vw->signalNewMrlReference(mrldata->mrl);
        break;
       }
     case XINE_EVENT_UI_MESSAGE:
       {
         kdDebug(555) << "xine message received" << endl;
         
         xine_ui_message_data_t *data = (xine_ui_message_data_t *)xineEvent->data;
         QString message;

         /* some code take from the xine-ui - Copyright (C) 2000-2003 the xine project */
         switch(data->type)
         {
           case XINE_MSG_NO_ERROR:
             {               
              /* copy strings, and replace '\0' separators by '\n' */
               char* s = data->messages;
               char* d = new char[2000];
     
               while(s && (*s != '\0') && ((*s + 1) != '\0'))
               {
                  switch(*s)
                  {
                    case '\0':
                      {
                         *d = '\n';
                         break;
                      }
                    default:
                      {
                         *d = *s;
                          break;
                      }
                  }   
                  s++;
                  d++;
               }
               *++d = '\0';

               message = d;
               delete d;
               break;
             }
           case XINE_MSG_GENERAL_WARNING:
            {
              message = i18n("General Warning: \n");

              if(data->explanation)
                message = message + ((char *) data + data->explanation) + " " + ((char *) data + data->parameters);
              else
                message = message + i18n("No Informations available.");

              break;
            }  
          case XINE_MSG_UNKNOWN_HOST:
            {
              message = i18n("The host you're trying to connect is unknown.\nCheck the validity of the specified hostname. ");
                if(data->explanation)
                  message = message + "(" + ((char *) data + data->parameters) + ")";
              break;
            }
          case XINE_MSG_UNKNOWN_DEVICE:
            {
              message = i18n("The device name you specified seems invalid. ");
              if(data->explanation)
                message = message + "(" + ((char *) data + data->parameters) + ")";
              break;
            }
          case XINE_MSG_NETWORK_UNREACHABLE:
            {
              message = i18n("The network looks unreachable.\nCheck your network setup and the server name. ");
              if(data->explanation)
                message = message + "(" + ((char *) data + data->parameters) + ")";
              break;
            }
          case XINE_MSG_CONNECTION_REFUSED:
            {
              message = i18n("The connection was refused.\nCheck the host name. ");
              if(data->explanation)
                message = message + "(" + ((char *) data + data->parameters) + ")";
              break;
            }
          case XINE_MSG_FILE_NOT_FOUND:
            {
              message = i18n("The specified file or url was not found. Please check it. ");
              if(data->explanation)
                message = message + "(" + ((char *) data + data->parameters) + ")";
              break;
            }  
          case XINE_MSG_READ_ERROR:
            {
              message = i18n("The source can't be read.\nMaybe you don't have enough rights for this, or source doesn't contain data (e.g: no disc in drive). ");
              if(data->explanation)
                message = message + "(" + ((char *) data + data->parameters) + ")";
              break;
            }  
          case XINE_MSG_LIBRARY_LOAD_ERROR:
            {
              message = i18n("A problem occur while loading a library or a decoder: ");
              if(data->explanation)
                message = message + ((char *) data + data->parameters);
              break;
            }  
          case XINE_MSG_ENCRYPTED_SOURCE:
            {
              message = i18n("The source seems encrypted, and can't be read. ");
              if (vw->currentTitle.contains("dvd:/"))
                message = message + i18n("\nYour DVD is probably crypted. According to your country laws, you can or can't use libdvdcss to be able to read this disc. ");
              if(data->explanation)
                message = message + "(" + ((char *) data + data->parameters) + ")";
              break;
            }
          default:
            {
              message = i18n("Unkown error: \n");
              if(data->explanation)
                message = message + ((char *) data + data->explanation) + " " + ((char *) data + data->parameters);
              break;
            }
          }

         vw->signalXineMessage( message );
         break;
       }
   }
}


/******************* new video driver *********************/

void VideoWindow::VideoDriverChangedCallback(void* p, xine_cfg_entry_t* entry)
{
  if (p == NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

  kdDebug(555) << "New video driver: " << entry->enum_values[entry->num_value] << endl;

  if (vw->filtersEnabled) vw->UnwireFilters();
      
  mutex.lock();

  bool playing = false;
  if (xine_get_status(vw->xineStream) == XINE_STATUS_PLAY)
   playing = true;

  xine_close(vw->xineStream);

  if (vw->xinePost)
  {
    xine_post_dispose (vw->xineEngine, vw->xinePost);
    vw->xinePost = NULL;
  }
  
  xine_event_dispose_queue(vw->eventQueue);
  xine_dispose(vw->xineStream);
  xine_close_video_driver(vw->xineEngine, vw->videoDriver);
  vw->videoDriver = NULL;

  vw->videoDriver = xine_open_video_driver(vw->xineEngine,
                           entry->enum_values[entry->num_value], XINE_VISUAL_TYPE_X11,
                           (void *) &(vw->visual));


  if (!vw->videoDriver)
  {
     QApplication::beep();
     vw->signalNewInfo( i18n("Error: Can't init new Video Driver %1 - using %2!").arg(entry->enum_values[entry->num_value]).arg(vw->videoDriverName) );
     playing = false;
     vw->videoDriver = xine_open_video_driver(vw->xineEngine,
                           vw->videoDriverName, XINE_VISUAL_TYPE_X11,
                           (void *) &(vw->visual));
  }
  else
  {
     vw->videoDriverName = entry->enum_values[entry->num_value];
     vw->signalNewInfo( i18n("Using Video Driver: ") + vw->videoDriverName );
  }

  vw->xineStream = xine_stream_new(vw->xineEngine, vw->audioDriver, vw->videoDriver);
  vw->eventQueue = xine_event_new_queue (vw->xineStream);
  xine_event_create_listener_thread(vw->eventQueue, &VideoWindow::XineEventListener, p);
  mutex.unlock();

  if (vw->filtersEnabled) vw->WireFilters();

  if (playing)
    vw->PlayMRL(vw->currentMRL, vw->currentTitle, false);
}


/*********************** new audio driver *************************/

void VideoWindow::AudioDriverChangedCallback(void* p, xine_cfg_entry_t* entry)
{
  if (p == NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

  kdDebug(555) << "New audio driver: " << entry->enum_values[entry->num_value] << endl;

  if (vw->filtersEnabled) vw->UnwireFilters();

  mutex.lock();

  bool playing = false;
  if (xine_get_status(vw->xineStream) == XINE_STATUS_PLAY)
   playing = true;

  xine_close(vw->xineStream);

  if (vw->xinePost)
  {
    xine_post_dispose (vw->xineEngine, vw->xinePost);
    vw->xinePost = NULL;
  }

  xine_event_dispose_queue(vw->eventQueue);
  xine_dispose(vw->xineStream);
  xine_close_audio_driver(vw->xineEngine, vw->audioDriver);
  vw->audioDriver = NULL;

  vw->audioDriver = xine_open_audio_driver(vw->xineEngine, entry->enum_values[entry->num_value], NULL);

  if (!vw->audioDriver)
  {
     QApplication::beep();
     vw->signalNewInfo( i18n("Error: Can't init new Audio Driver %1 - using %2!").arg(entry->enum_values[entry->num_value]).arg(vw->audioDriverName) );
     playing = false;
     vw->audioDriver = xine_open_audio_driver(vw->xineEngine, vw->audioDriverName, NULL);
  }
  else
  {
     vw->audioDriverName = entry->enum_values[entry->num_value];
     vw->signalNewInfo( i18n("Using Audio Driver: ") + vw->audioDriverName );
  }

  vw->xineStream = xine_stream_new(vw->xineEngine, vw->audioDriver, vw->videoDriver);
  vw->eventQueue = xine_event_new_queue (vw->xineStream);
  xine_event_create_listener_thread(vw->eventQueue, &VideoWindow::XineEventListener, p);
  mutex.unlock();

  if (vw->filtersEnabled) vw->WireFilters();

  if (playing)
    vw->PlayMRL(vw->currentMRL, vw->currentTitle, false);
}
    

/************ change path to logo file *************/
    
void VideoWindow::LogoPathChangedCallback(void* p, xine_cfg_entry_t* entry)
{
  if (p==NULL) return;
  VideoWindow* vw = (VideoWindow*) p;

  vw->logoPath = entry->str_value;
}  


/**********************************************
 *      EVENT LOOP
 *********************************************/

void* VideoWindow::EventLoop(void* p)
{
  if (p == NULL) return NULL;
  VideoWindow* vw = (VideoWindow*) p;

  kdDebug(555) << "Start event loop...\n";

  XEvent event;

 while(1)
 {
    XNextEvent (vw->xineDisplay, &event);

    if (event.type == Expose)
    {
      if (event.xexpose.count == 0)
      {
        mutex.lock();
        xine_gui_send_vo_data (vw->xineStream, XINE_GUI_SEND_EXPOSE_EVENT, &event);
        mutex.unlock();
      }
    }
        
    if (event.type == ClientMessage)
    {
      XClientMessageEvent *cm = (XClientMessageEvent *) &event;

      if (cm->message_type == vw->atomQuit)
       break;
    }
  }

  kdDebug(555) << "Exiting event loop...\n";
  pthread_exit(0);
  return NULL;
}    
  


/**********************************************************
 *           INIT XINE ENGINE
 *********************************************************/

void VideoWindow::polish()
{
  if (!startXineManual)  /* start xine engine automatically? */
  {
    if (!InitXine()) emit signalQuit(); /* no success */
  }  
}    

 
bool VideoWindow::InitXine()
{
 if (xineRunning) return true;
 
 bool cmdlAudioEnum = false, cmdlVideoEnum = false;

 GlobalPosChanged(); /* get global pos of the window */
   
/**** INIT XINE DISPLAY  ****/
 

  if (!XInitThreads ())
  {
    QApplication::beep();
    KMessageBox::error(0, i18n("XInitThreads failed!"));
    return false;
  }

  xineDisplay = XOpenDisplay( getenv("DISPLAY") );

  if (!xineDisplay)
  {
    QApplication::beep();
    KMessageBox::error(0, i18n("Failed to connect to X-Server!"));
    return false;
  }

  xineScreen = DefaultScreen( xineDisplay );
  //xineScreen = x11Screen();
  xineWindow = winId();      

  LockXineDisplay();      
/* create a simple window for comunication only */

  xComWindow = XCreateSimpleWindow( xineDisplay, DefaultRootWindow(xineDisplay), 0, 0, 1, 1, 0, 0, 0 );

  XSelectInput( xineDisplay, xineWindow, ExposureMask );

/* determine display aspect ratio  */

  double resHor = DisplayWidth( xineDisplay, xineScreen )*1000 / DisplayWidthMM( xineDisplay, xineScreen );
  double resVer = DisplayHeight( xineDisplay, xineScreen )*1000 / DisplayHeightMM( xineDisplay, xineScreen );

  displayRatio = resVer / resHor;
  //if (QABS( displayRatio - 1.0 ) < 0.01) displayRatio   = 1.0;

  kdDebug(555) << "Display aspect ratio (v/h): " << displayRatio << endl;

  atomQuit = XInternAtom(xineDisplay, "QUIT_SIGNAL", False );

  UnlockXineDisplay();

/**** INIT XINE ENGINE ****/

  kdDebug(555) << "Using xine version " << xine_get_version_string() << "\n";

  xineEngine = xine_new();
  if (!xineEngine)
  {
    QApplication::beep();
    KMessageBox::error(0, i18n("Can't init Xine Engine!"));
    return false;
  }  

/* load configuration */

  configFile = QDir::homeDirPath();
  configFile.append("/.kaffeine/config");

  if (!QFile::exists(configFile))
    kdWarning(555) << "No config file found, will create one...\n";
  else
    xine_config_load (xineEngine, configFile);


   kdDebug(555) << "Post-init xine-engine\n";
   xine_init(xineEngine);

/** set xine parameters **/

  const char* const* drivers = NULL;
  char** audioChoices = NULL;
  int i = 0;

  drivers = xine_list_audio_output_plugins (xineEngine);
  audioChoices = new char*[15];
  for(i = 0; i < 15; i++) audioChoices[i] = new char[10];
  audioChoices[0] = const_cast<char*>("auto");
  i = 0;
  while(drivers[i])
  {
    audioChoices[i+1] = (char*)drivers[i];
    if (preferedAudio == drivers[i]) cmdlAudioEnum = true;
    i++;
  }  
  audioChoices[i+1] = NULL;
  if (preferedAudio == "auto") cmdlAudioEnum = true;
  
  char* audioInfo = new char[200];
  strcpy(audioInfo, i18n("Audiodriver to use (default: auto)").local8Bit() );
  i = xine_config_register_enum(xineEngine, "gui.audiodriver", 0,
                   audioChoices, audioInfo, NULL, 10, &VideoWindow::AudioDriverChangedCallback, this);

  if (cmdlAudioEnum)
    audioDriverName = preferedAudio;
   else 
    audioDriverName = audioChoices[i];

  kdDebug(555) << "Use audio driver " << audioDriverName << "\n";

  i = 0;
  char** videoChoices = NULL;
  drivers = xine_list_video_output_plugins (xineEngine);
  videoChoices = new char*[15];
  for(i = 0; i < 15; i++) videoChoices[i] = new char[10];
  i = 0;
  videoChoices[0] = const_cast<char*>("auto");
  while(drivers[i])
  {
    videoChoices[i+1] = (char*)drivers[i];
    if (preferedVideo == drivers[i]) cmdlVideoEnum = true;
    i++;
  }
  videoChoices[i+1] = NULL;
  if (preferedVideo == "auto") cmdlVideoEnum = true;

  char* videoInfo = new char[200];
  strcpy( videoInfo, i18n("Videodriver to use (default: auto)").local8Bit() );
  i = xine_config_register_enum(xineEngine, "gui.videodriver", 0,
                   videoChoices, videoInfo, NULL, 10, &VideoWindow::VideoDriverChangedCallback, this);

  if (cmdlVideoEnum)
    videoDriverName = preferedVideo;
   else 
    videoDriverName = videoChoices[i];

  kdDebug(555) << "Use video driver " << videoDriverName << "\n";

  

/* init video driver */
  visual.display          = xineDisplay;
  visual.screen           = xineScreen;
  visual.d                = xineWindow;
  visual.dest_size_cb     = &VideoWindow::DestSizeCallback;
  visual.frame_output_cb  = &VideoWindow::FrameOutputCallback;
  visual.user_data        = (void*)this;

  kdDebug(555) << "Init video driver\n";

  videoDriver = xine_open_video_driver(xineEngine,
               videoDriverName,  XINE_VISUAL_TYPE_X11,
               (void *) &(visual));
  if (!videoDriver)
  {
    QApplication::beep();
    KMessageBox::error(0, i18n("Can't init Video Driver!") + " (" + videoDriverName + ")");
    return false;
  }

/* init audio driver */  
  kdDebug(555) << "Init audio driver\n";
  
  audioDriver = xine_open_audio_driver (xineEngine, audioDriverName, NULL);
  if (!audioDriver)
  {
    QApplication::beep();
    KMessageBox::error(0, i18n("Can't init Audio Driver!")+ " (" + audioDriverName + ")");
    return false;
  }  

//kdDebug(555) << "Open stream\n";
  
  xineStream  = xine_stream_new(xineEngine, audioDriver, videoDriver);
  if (!xineStream)
  {
    QApplication::beep();
    KMessageBox::error(0, i18n("Can't create a new Xine-Stream!"));
    return false;
  }

/** event handling **/

  eventQueue = xine_event_new_queue (xineStream);
  xine_event_create_listener_thread(eventQueue, &VideoWindow::XineEventListener, (void*)this);
  
/** start event thread **/

   if (pthread_create(&eventThread, NULL, &VideoWindow::EventLoop, this))
   {
     QApplication::beep();
     KMessageBox::error(0, i18n("Can't create thread!"));
     return false;
   }

/* set path to logo file */

  QStringList dirs = KGlobal::dirs()->findDirs("data","kaffeine");
  logoPath = dirs[0];  // global kde path
  logoPath.append("logo.avi");

 // kdDebug(555) << "Logopath: " << logoPath << "\n";

  char* logoInfo = new char[200];
  strcpy( logoInfo, i18n("Absolute path to the logo URL (png file for example)").local8Bit() );
  logoPath = xine_config_register_string(xineEngine, "gui.logo_path", logoPath,
                     logoInfo, NULL, 10,
                     &VideoWindow::LogoPathChangedCallback, this);

/** get a keycode for a faked key event to hold down screensaver **/

#ifdef HAVE_XTEST
int a,b,c,d;
  haveXTest = XTestQueryExtension(x11Display(), &a, &b, &c, &d);
  if (haveXTest)
    xTestKeycode = XKeysymToKeycode(x11Display(), XK_Shift_L);
  
#endif

       
  kdDebug(555) << "xine init successful\n";
  xineRunning = true;

  parentWidget()->polish(); /* make sure gui will build correctly */
  
  return true;
}



/************************************************
 *              PLAY MRL
 ************************************************/


bool VideoWindow::PlayMRL(const QString& mrl, const QString& title, bool returnInfo)
{
  mutex.lock();
  
  if (xine_get_status(xineStream) == XINE_STATUS_PLAY)
    xine_stop(xineStream);

 
  emit signalNewInfo(i18n("Opening..."));
  setCursor(QCursor(Qt::WaitCursor));

  currentMRL = mrl;

 /* check for external subtitle file or save url */
  extraInfo = QString::null;
  QString ref;
  
  for (int i = 1; i <= mrl.contains('#'); i++)
  {
    ref = mrl.section('#', i, i);
    if (ref.section(':', 0, 0) == "subtitle")
      extraInfo = extraInfo + " ** " + i18n("Subtitles: ") + ref.section(':', 1);
    if (ref.section(':', 0, 0) == "save")
      extraInfo = extraInfo + " ** " + i18n("Save as: ") + ref.section(':', 1);
  }
 
  kdDebug(555) << "Playing: " << currentMRL.local8Bit() << endl;

  if (!xine_open(xineStream, currentMRL.local8Bit() )) /** pass mrl local 8Bit encoded **/
  {

    int errCode = xine_get_error(xineStream);
    if ( (errCode < 0) || (errCode > 5) ) errCode = 6;
    
    QString err = xineErrors[errCode];
    emit signalNewInfo(err);

    if (returnInfo)
    {
      emit signalStreamInfo(err);
      emit signalLengthInfo(i18n("*Error*"));
    }  
    setCursor(QCursor(Qt::ArrowCursor));
    mutex.unlock();
    return false;
  }  

/**** use visualization ? ****/

 if (!xine_get_stream_info (xineStream, XINE_STREAM_INFO_HAS_VIDEO))
 {

   if (visualPluginName && (!xinePost))
   {
    xinePost = xine_post_init (xineEngine, visualPluginName, 0,
          &audioDriver,
          &videoDriver);

     postAudioSource = xine_get_audio_source (xineStream);
     postInput = (xine_post_in_t*)xine_post_input (xinePost, const_cast<char*>("audio in"));
     xine_post_wire (postAudioSource, postInput);
   }
  }
  else
  {
    if (xinePost)
    {
      postAudioSource = xine_get_audio_source (xineStream);
      xine_post_wire_audio_port (postAudioSource, audioDriver);
      xine_post_dispose (xineEngine, xinePost);
      xinePost = NULL;
    }
   }

/*** play ***/

  if (!xine_play(xineStream, 0,0))
  {
    int errCode = xine_get_error(xineStream);
    if ( (errCode < 0) || (errCode > 5) ) errCode = 6;
    
    QString err = xineErrors[errCode];
    emit signalNewInfo(err);
    if (returnInfo)
    {
      emit signalStreamInfo(err);
      emit signalLengthInfo(i18n("*Error*"));
    }  
    setCursor(QCursor(Qt::ArrowCursor));
    mutex.unlock();
    return false;
  }

/* do the stream have chapters ? */
  if (xine_get_stream_info(xineStream, XINE_STREAM_INFO_HAS_CHAPTERS))
    emit signalHasChapters(true);
   else
    emit signalHasChapters(false);
 
/** information requirement **/

  currentTitle = title;

  if ( (title.contains('.')) || (mrl.contains(':')) ) /* tags was still read  || !local file -- FIXME*/
  {
    QString metaTitle = QString::fromLocal8Bit( xine_get_meta_info(xineStream, XINE_META_INFO_TITLE) );
    if ( (metaTitle) && (!metaTitle.isEmpty()) )  //usable meta info
    {
      QString metaArtist = QString::fromLocal8Bit(xine_get_meta_info(xineStream, XINE_META_INFO_ARTIST));
      QString metaAlbum = QString::fromLocal8Bit(xine_get_meta_info(xineStream, XINE_META_INFO_ALBUM));
      QString metaInfo = QString(metaString).arg(metaArtist).arg(metaTitle).arg(metaAlbum);
      currentTitle = metaInfo;
      if ((metaInfo != title) && (returnInfo))
        emit signalMetaInfo(metaInfo);
    }
  }  

  mutex.unlock();
  
  trackInfoRow = currentTitle;

  QString streamInfo = GetStreamInfo();
  if (returnInfo)
  {
     emit signalStreamInfo(streamInfo);
  }

  streamInfo = streamInfo + extraInfo;
                                                                                                        
  trackInfoRow.append( streamInfo.prepend(" ** ") );
  emit signalNewInfo(trackInfoRow);
   
  slotSetAudioChannel(0); //refresh channel info

  posTimer.start(500);

/* wait for available track length info */
  if (returnInfo)
  {
    lengthInfoTimer.start(1000, true); //single shot
  }  
    
  setCursor(QCursor(Qt::ArrowCursor));
                
  emit signalShowOSD(currentTitle);

  return true;
}




/****************** postprocessing filter management ****************/

void VideoWindow::GetPostprocessFilterNames( QStringList& filters )
{
  const char* const* plugins = xine_list_post_plugins_typed(xineEngine, XINE_POST_TYPE_VIDEO_FILTER);

  for (int i = 0; plugins[i]; i++)
  {
    filters << plugins[i];
  }  
}


void VideoWindow::slotCreateFilter( const QString& name, QWidget* parent )
{
  if (filtersEnabled) UnwireFilters();

  PostFilter* filter = new PostFilter( name, xineEngine, audioDriver, videoDriver, parent );
  connect( filter, SIGNAL( signalDeleteMe( PostFilter* ) ), this, SLOT( slotDeleteFilter( PostFilter* ) ));
  filterList.append( filter );

  if (filtersEnabled) WireFilters();
}  
  

void VideoWindow::slotRemoveAllFilters()
{
  if (!filterList.count()) return;

  if (filtersEnabled) UnwireFilters();
  while (filterList.count())
    filterList.removeLast();
}  
  
  
void VideoWindow::slotDeleteFilter( PostFilter* filter )
{
  if (filtersEnabled) UnwireFilters();
  filterList.remove( filter );
  if (filtersEnabled) WireFilters();
}


void VideoWindow::UnwireFilters()
{
  if (!filterList.count()) return;

  mutex.lock();
  xine_post_wire_video_port(  xine_get_video_source( xineStream), videoDriver );
  mutex.unlock();
  /* Waiting a small chunk of time helps to avoid crashing */
  xine_usec_sleep(500000);
}


void VideoWindow::WireFilters()
{
  if (!filterList.count()) return;

  mutex.lock();
  xine_post_wire_video_port( filterList.at( filterList.count()-1 )->GetOutput(), videoDriver );

  for (uint i = filterList.count()-1; i >0; i--)
  {
    xine_post_wire( filterList.at( i-1 )->GetOutput(), filterList.at( i )->GetInput() );
  }  

  xine_post_wire( xine_get_video_source( xineStream ), filterList.at( 0 )->GetInput() );
  mutex.unlock();
}  


void VideoWindow::slotEnableFilters(bool enable)
{
  filtersEnabled = enable;
  if (enable)
     WireFilters();
    else
     UnwireFilters();
}     
    

/****  visual plugin **********/


void VideoWindow::GetVisualPlugins(QStringList& visuals) const
{
  const char* const* plugins = xine_list_post_plugins_typed(xineEngine, XINE_POST_TYPE_AUDIO_VISUALIZATION);

  for (int i = 0; plugins[i]; i++)
  {
    visuals << plugins[i];
  }
}   


/**************** change visualization plugin *****************/

void VideoWindow::SetVisualPlugin(const QString& visual)
{
  if ( visualPluginName == visual ) return;
  
  kdDebug(555) << "New visualization plugin: " << visual << endl;

  if (visual == "none")
    visualPluginName = QString::null;
   else
    visualPluginName = visual;

  mutex.lock();

  if (xinePost)
  {
    xine_post_out_t *pp;

    pp = xine_get_audio_source (xineStream);
    xine_post_wire_audio_port (pp, audioDriver);
    xine_post_dispose (xineEngine, xinePost);
    xinePost = NULL;
  }


  
  if ( (xine_get_status( xineStream ) == XINE_STATUS_PLAY)
       && (!xine_get_stream_info(xineStream, XINE_STREAM_INFO_HAS_VIDEO)) && (visualPluginName) )
  {

    xinePost = xine_post_init (xineEngine, visualPluginName, 0, &audioDriver, &videoDriver);
    postAudioSource = xine_get_audio_source(xineStream);
    postInput = (xine_post_in_t*)xine_post_input(xinePost, const_cast<char*>("audio in"));
    xine_post_wire( postAudioSource, postInput );
  }

  mutex.unlock();
}


/*****/


void VideoWindow::GetAutoplayPlugins(QStringList& autoPlayList) const
{
  char** pluginIds = NULL;
  int i = 0;

  mutex.lock();
  pluginIds = (char**)xine_get_autoplay_input_plugin_ids(xineEngine);

  while(pluginIds[i])
  {
    autoPlayList << pluginIds[i];
    
    autoPlayList << xine_get_input_plugin_description(xineEngine, pluginIds[i]);    
    i++;
  }
  mutex.unlock(); 
}

 
bool VideoWindow::GetAutoplayPluginMrl(const QString& plugin, QStringList& list)
{
  char** mrls = NULL;
  int num;
  int i = 0;

  mutex.lock();
  mrls = xine_get_autoplay_mrls(xineEngine, plugin, &num);
  mutex.unlock();
  if (mrls)
  {
    while (mrls[i])
      {
        list << mrls[i];
        i++;
      }  

    return true;
   } 
   else 
   {
     QString error(i18n("Error: No "));
     error.append(plugin);
     error.append(i18n(", or wrong path to device."));
     signalNewInfo(error);
     return false;
   }
}


int VideoWindow::GetVolume() const
{
  return xine_get_param(xineStream, XINE_PARAM_AUDIO_VOLUME);
}


void VideoWindow::slotSetVolume(int vol)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_AUDIO_VOLUME, -vol);
  mutex.unlock();
}


void VideoWindow::mouseMoveEvent(QMouseEvent* mev)
{
   if (!xineRunning) return;

   if (cursor().shape() == Qt::BlankCursor)
     setCursor(QCursor(Qt::ArrowCursor));

    x11_rectangle_t   rect;
    xine_event_t      event;
    xine_input_data_t input;

    rect.x = mev->x();
    rect.y = mev->y();
    rect.w = 0;
    rect.h = 0;

    mutex.lock();
    xine_gui_send_vo_data (xineStream,
               XINE_GUI_SEND_TRANSLATE_GUI_TO_VIDEO,
                (void*)&rect);

    event.type        = XINE_EVENT_INPUT_MOUSE_MOVE;
    event.data        = &input;
    event.data_length = sizeof(input);
    input.button      = 0; 
    input.x           = rect.x;
    input.y           = rect.y;
    xine_event_send (xineStream, &event);
    mutex.unlock();
}


void VideoWindow::mousePressEvent(QMouseEvent* mev)
{
   if (!xineRunning) return;

   if (mev->button() == Qt::RightButton)
   {
     emit signalShowContextMenu(mev->globalPos());
     return;
   }  

   if (mev->button() == Qt::LeftButton)
   {

    int cur = cursor().shape();
    if ( (cur == Qt::ArrowCursor) || (cur == Qt::BlankCursor) )
    {
      emit signalShowFullscreenPanel(mev->globalPos());
      return;
    }  

    x11_rectangle_t   rect;
    xine_event_t      event;
    xine_input_data_t input;

    rect.x = mev->x();
    rect.y = mev->y();
    rect.w = 0;
    rect.h = 0;

    mutex.lock();
    xine_gui_send_vo_data (xineStream,
                XINE_GUI_SEND_TRANSLATE_GUI_TO_VIDEO,
                   (void*)&rect);

    event.type        = XINE_EVENT_INPUT_MOUSE_BUTTON;
    event.data        = &input;
    event.data_length = sizeof(input);
    input.button      = 1;
    input.x           = rect.x;
    input.y           = rect.y;
    xine_event_send (xineStream, &event);
    mutex.unlock();
   }
}


void VideoWindow::PlayNextChapter() const
{

  xine_event_t xev;

  xev.type = XINE_EVENT_INPUT_NEXT;
  xev.data = NULL;
  xev.data_length = 0;

  mutex.lock();
  xine_event_send(xineStream, &xev);
  mutex.unlock();
}  


void VideoWindow::PlayPreviousChapter() const
{

  xine_event_t xev;

  xev.type = XINE_EVENT_INPUT_PREVIOUS;
  xev.data = NULL;
  xev.data_length = 0;

  mutex.lock();
  xine_event_send(xineStream, &xev);
  mutex.unlock();
}  


void VideoWindow::slotStopPlayback()
{
    posTimer.stop();
    PlayLOGO();
}


void VideoWindow::PlayLOGO() 
{
  mutex.lock();
  if (xine_get_status(xineStream) == XINE_STATUS_PLAY)
    xine_stop(xineStream);

  if (!logoPath.isNull())
   {
   
     if(!xine_open(xineStream, logoPath))
     {
       kdWarning(555) << "Invalid logo file\n";
       logoPath = QString::null;
       mutex.unlock();
       return;
     }  

     xine_play(xineStream, 0,0);
   }
  mutex.unlock();
}


void VideoWindow::SetDevice(const QString& device)
{
  kdDebug(555) << "Set device to" << device << "\n";

  xine_cfg_entry_t config;

  xine_config_lookup_entry (xineEngine, "input.cdda_device", &config);
  config.str_value = (char*)device.latin1();
  xine_config_update_entry (xineEngine, &config);

  xine_config_lookup_entry (xineEngine, "input.vcd_device", &config);
  config.str_value = (char*)device.latin1();
  xine_config_update_entry (xineEngine, &config);
}


void VideoWindow::SetStreamSaveDir(const QString& dir)
{
  xine_cfg_entry_t config;

  if (!xine_config_lookup_entry (xineEngine, "misc.save_dir", &config)) return; /* older lib */

  kdDebug(555) << "Set misc.save_dir to: " << dir << endl;
  config.str_value = (char*)dir.latin1();
  xine_config_update_entry (xineEngine, &config);
}  


void VideoWindow::SetBroadcasterPort(const uint port)
{
  kdDebug(555) << "Set broadcaster port to " << port << endl;
  xine_set_param(xineStream, XINE_PARAM_BROADCASTER_PORT, port);
}  


void VideoWindow::slotSpeedPause()
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
  mutex.unlock();
  posTimer.stop();
  signalNewInfo("Pause.");
}


void VideoWindow::slotSpeedNormal()
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
  mutex.unlock();
  posTimer.start(500);
  signalNewInfo(trackInfoRow);
}


QString VideoWindow::GetMimeTypes() const
{  
  QString mime( xine_get_mime_types(xineEngine) );
  return mime;
}


void VideoWindow::slotSetAudioChannel(int ch)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL, ch-1);
  mutex.unlock();
}  


void VideoWindow::slotSetSubtitleChannel(int ch)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_SPU_CHANNEL, ch-1);
  mutex.unlock();
}


void VideoWindow::slotTogglePlayMode()
{
  switch (playMode)
 {
   case NORMAL_PLAY:
   {
     playMode = REPEAT_PLAY;
     break;
   }
   case REPEAT_PLAY:
   {
     playMode = PERCENT_PLAY;
     break;
   }
   case PERCENT_PLAY:
   {
     playMode = NORMAL_PLAY;
     break;
   }
 }
}


QString VideoWindow::GetTimeString(int msec)
{
  int hours;
  int min;
  int sec;
  QString tmp;
  QString t;

    msec = msec/1000;  //sec
    hours = msec/3600;
    t = t.setNum(hours);
    t.append(":");

    msec = msec - (hours*3600);
    min = msec / 60;
    tmp = tmp.setNum(min);
    tmp = tmp.rightJustify(2, '0');
    t.append(tmp);
    t.append(":");

    sec = msec - (min*60);
    tmp = tmp.setNum(sec);
    tmp = tmp.rightJustify(2, '0');
    t.append(tmp);

   return t;                                                                       
}


void VideoWindow::slotGetPosition()
{
  if (!xineRunning) return;

  int pos;
  int time;
  int length;
  
  double perc;

  QString t;
  
  mutex.lock();
  if ( !xine_get_pos_length(xineStream, &pos, &time, &length) )
  {
    kdDebug(555) << "No valid stream position information" << endl;
    mutex.unlock();
    return;
  }  
  mutex.unlock();
  
  switch (playMode)
  { 
    case PERCENT_PLAY:
    {
      perc = pos/655,35;
      time = (int) (perc);
      if ( (time < 0) || (time > 100) )
      {
        emit signalNewPosition( pos , "  --%  " );
        return;
      }  
      t = t.setNum(time);
      t = QString(" %1%2 ").arg(t).arg("%");
      emit signalNewPosition( pos, t );
      break;
    }  
    case REPEAT_PLAY:
    {
      time = length - time;      
    }
    case NORMAL_PLAY:
    {
      if (time < 0)
      {
        emit signalNewPosition( pos , "-:--:--" );
        return;
      } 
      emit signalNewPosition( pos, GetTimeString(time) );
      break;
    }
  }
}


void VideoWindow::slotChangePosition(int pos)
{
  if (!xineRunning) return;

  mutex.lock();
  int pause = !xine_get_param(xineStream, XINE_PARAM_SPEED);
  if ( (xine_get_status(xineStream) == XINE_STATUS_PLAY) && (xine_get_stream_info(xineStream, XINE_STREAM_INFO_SEEKABLE)) )
  {
    xine_play(xineStream, pos, 0);
  }
  mutex.unlock();
  if (pause)
    slotSpeedPause();
}


void VideoWindow::slotEject()
{
  mutex.lock();
  xine_eject(xineStream);
  mutex.unlock();
}


void VideoWindow::slotToggleDeinterlace()
{
  mutex.lock();
  if (xine_get_param(xineStream, XINE_PARAM_VO_DEINTERLACE))
    {
     xine_set_param(xineStream, XINE_PARAM_VO_DEINTERLACE, false);
   }  
   else
    xine_set_param(xineStream, XINE_PARAM_VO_DEINTERLACE, true);

  mutex.unlock();  
}


void VideoWindow::slotAspectRatioAuto()
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_VO_ASPECT_RATIO, XINE_VO_ASPECT_AUTO);
  mutex.unlock();
}  


void VideoWindow::slotAspectRatio4_3()
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_VO_ASPECT_RATIO, XINE_VO_ASPECT_4_3);
  mutex.unlock();
}


void VideoWindow::slotAspectRatio16_9()
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_VO_ASPECT_RATIO, XINE_VO_ASPECT_ANAMORPHIC);
  mutex.unlock();
}


void VideoWindow::slotAspectRatioSquare()
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_VO_ASPECT_RATIO, XINE_VO_ASPECT_SQUARE);
  mutex.unlock();
}


void VideoWindow::slotZoomOut()
{
  if ((currentZoom - 5) >= 100)
  {
    currentZoom -= 5;
    mutex.lock();
    xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_X, currentZoom);
    xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_Y, currentZoom);
    mutex.unlock();
  }
}


void VideoWindow::slotZoomIn()
{
  if ((currentZoom + 5) <= XINE_VO_ZOOM_MAX)
  {
    currentZoom += 5;
    mutex.lock();
    xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_X, currentZoom);
    xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_Y, currentZoom);
    mutex.unlock();
  }
}


void VideoWindow::slotZoomOff()
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_X, 100);
  xine_set_param(xineStream, XINE_PARAM_VO_ZOOM_Y, 100);
  mutex.unlock();
  currentZoom = 100;
}


QString VideoWindow::GetStreamInfo()
{
  QString streamInfo;

  mutex.lock();
  streamInfo = streamInfo + "(" + xine_get_meta_info(xineStream, XINE_META_INFO_INPUT_PLUGIN) + ") ";

  if (xine_get_stream_info(xineStream, XINE_STREAM_INFO_HAS_VIDEO))
  {
     streamInfo.append(xine_get_meta_info(xineStream, XINE_META_INFO_VIDEOCODEC));
     streamInfo.append(" (");
     streamInfo.append( QString::number(xine_get_stream_info(xineStream, XINE_STREAM_INFO_VIDEO_WIDTH)) );
     streamInfo.append("x");
     streamInfo.append( QString::number(xine_get_stream_info(xineStream, XINE_STREAM_INFO_VIDEO_HEIGHT)) );
     streamInfo.append(") ");
  }

  if (xine_get_stream_info(xineStream, XINE_STREAM_INFO_HAS_AUDIO))
  {
     streamInfo.append(xine_get_meta_info(xineStream, XINE_META_INFO_AUDIOCODEC));
     streamInfo.append(" (");
     streamInfo.append( QString::number(xine_get_stream_info(xineStream, XINE_STREAM_INFO_AUDIO_BITRATE)/1000) );
     streamInfo.append("kb)");
  }

  mutex.unlock();
  return streamInfo;
}


QString VideoWindow::GetLengthInfo()
{
  int pos, time, length;

  mutex.lock();
  bool ok = xine_get_pos_length(xineStream, &pos, &time, &length);
  mutex.unlock();

  if ( (ok) && (length > 0) )
  {
    return GetTimeString(length);
  }
  
  return QString::null;
}  


void VideoWindow::slotEmitLengthInfo()
{
  emit signalLengthInfo( GetLengthInfo() );  
}


void VideoWindow::slotSetConfig(const QString& mstr, int ssTimeout)
{
  metaString = mstr;
  screensaverTimeout = ssTimeout;

#ifdef HAVE_XTEST
  if (screensaverTimeout > 0)
    screensaverTimer.start(screensaverTimeout* 60000); //min
   else
    screensaverTimer.stop();
#endif    
}


QString VideoWindow::GetMetaString() const
{
  return this->metaString;
}

int VideoWindow::GetScreensaverTimeout() const
{
  return screensaverTimeout;
}


void VideoWindow::SetXineVerbosityDebug()
{
  xine_set_param( xineStream, XINE_PARAM_VERBOSITY, XINE_VERBOSITY_DEBUG );
}


void VideoWindow::GlobalPosChanged()
{
  QPoint g = mapToGlobal(QPoint(0,0));
  globX = g.x();
  globY = g.y();
//  kdDebug(555) << "VideoWindow: x: " << globX << " y: " << globY << endl;
}    


void VideoWindow::slotFakeKeyEvent()
{
#ifdef HAVE_XTEST

// kdDebug(555) << "Fake key event...\n";

 if (haveXTest)
 {
   XTestFakeKeyEvent(x11Display(), xTestKeycode, true, CurrentTime);
   XTestFakeKeyEvent(x11Display(), xTestKeycode, false, CurrentTime);
   XSync(x11Display(), false);
 }  

#endif 

}


const xine_t* const VideoWindow::GetXineEngine()const
{
  return xineEngine;
}

  
/************ video settings ****************/

void VideoWindow::GetVideoSettings(int& hue, int& sat, int& contrast, int& bright,
                                   int& audioAmp, int& avOffset, int& spuOffset) const
{
  mutex.lock();
  hue = xine_get_param(xineStream, XINE_PARAM_VO_HUE);
  sat = xine_get_param(xineStream, XINE_PARAM_VO_SATURATION);
  contrast = xine_get_param(xineStream, XINE_PARAM_VO_CONTRAST);
  bright = xine_get_param(xineStream, XINE_PARAM_VO_BRIGHTNESS);

  audioAmp = xine_get_param(xineStream, XINE_PARAM_AUDIO_AMP_LEVEL);
  avOffset = xine_get_param(xineStream, XINE_PARAM_AV_OFFSET);
  spuOffset = xine_get_param(xineStream, XINE_PARAM_SPU_OFFSET);
  mutex.unlock();
}


void VideoWindow::slotSetHue(int hue)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_VO_HUE, hue);
  mutex.unlock();
}
  

void VideoWindow::slotSetSaturation(int sat)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_VO_SATURATION, sat);
  mutex.unlock();
}
  

void VideoWindow::slotSetContrast(int contrast)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_VO_CONTRAST, contrast);
  mutex.unlock();
}
  

void VideoWindow::slotSetBrightness(int bright)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_VO_BRIGHTNESS, bright);
  mutex.unlock();
}
  

void VideoWindow::slotSetAudioAmp(int amp)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_AUDIO_AMP_LEVEL, amp);
  mutex.unlock();
}
  

void VideoWindow::slotSetAVOffset(int av)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_AV_OFFSET, av);
  mutex.unlock();
}
  

void VideoWindow::slotSetSpuOffset(int spu)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_SPU_OFFSET, spu);
  mutex.unlock();
}



/**************** equalizer *****************/

void VideoWindow::slotSetEq30(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_30HZ, -val);
  mutex.unlock();
}


void VideoWindow::slotSetEq60(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_60HZ, -val);
  mutex.unlock();
}


void VideoWindow::slotSetEq125(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_125HZ, -val);
  mutex.unlock();
}


void VideoWindow::slotSetEq250(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_250HZ, -val);
  mutex.unlock();
}


void VideoWindow::slotSetEq500(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_500HZ, -val);
  mutex.unlock();
}


void VideoWindow::slotSetEq1k(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_1000HZ, -val);
  mutex.unlock();
}


void VideoWindow::slotSetEq2k(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_2000HZ, -val);
  mutex.unlock();
}


void VideoWindow::slotSetEq4k(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_4000HZ, -val);
  mutex.unlock();
}


void VideoWindow::slotSetEq8k(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_8000HZ, -val);
  mutex.unlock();
}


void VideoWindow::slotSetEq16k(int val)
{
  mutex.lock();
  xine_set_param(xineStream, XINE_PARAM_EQ_16000HZ, -val);
  mutex.unlock();
}

/*************** dvd menus ******************/

void VideoWindow::slotMenu1()
{
  xine_event_t xev;
  xev.type = XINE_EVENT_INPUT_MENU1;
  xev.data = NULL;
  xev.data_length = 0;

  mutex.lock();
  xine_event_send(xineStream, &xev);
  mutex.unlock();
}

void VideoWindow::slotMenu2()
{
  xine_event_t xev;
  xev.type = XINE_EVENT_INPUT_MENU2;
  xev.data = NULL;
  xev.data_length = 0;

  mutex.lock();
  xine_event_send(xineStream, &xev);
  mutex.unlock();
}

void VideoWindow::slotMenu3()
{
  xine_event_t xev;
  xev.type = XINE_EVENT_INPUT_MENU3;
  xev.data = NULL;
  xev.data_length = 0;

  mutex.lock();
  xine_event_send(xineStream, &xev);
  mutex.unlock();
}


void VideoWindow::slotMenu4()
{
  xine_event_t xev;
  xev.type = XINE_EVENT_INPUT_MENU4;
  xev.data = NULL;
  xev.data_length = 0;

  mutex.lock();
  xine_event_send(xineStream, &xev);
  mutex.unlock();
}


void VideoWindow::slotMenu5()
{
  xine_event_t xev;
  xev.type = XINE_EVENT_INPUT_MENU5;
  xev.data = NULL;
  xev.data_length = 0;

  mutex.lock();
  xine_event_send(xineStream, &xev);
  mutex.unlock();
}


void VideoWindow::slotMenu6()
{
  xine_event_t xev;
  xev.type = XINE_EVENT_INPUT_MENU6;
  xev.data = NULL;
  xev.data_length = 0;

  mutex.lock();
  xine_event_send(xineStream, &xev);
  mutex.unlock();
}


void VideoWindow::slotMenu7()
{
  xine_event_t xev;
  xev.type = XINE_EVENT_INPUT_MENU7;
  xev.data = NULL;
  xev.data_length = 0;

  mutex.lock();
  xine_event_send(xineStream, &xev);
  mutex.unlock();
}



/******** mouse hideing at fullscreen ****/


void VideoWindow::StartMouseHideTimer()
{
  mouseHideTimer.start(4000);
}
  

void VideoWindow::StopMouseHideTimer()
{
  mouseHideTimer.stop();
}
  

void VideoWindow::slotHideMouse()
{
  if (cursor().shape() == Qt::ArrowCursor)
   setCursor(QCursor(Qt::BlankCursor));
}   
  

/************************************************************
 *   Take a Screenshot                                      *
 *                                                          *
 *   code from gxine modified for 32bit output              *
 *   Copyright (C) 2000-2003 the xine project               *
 ************************************************************/

void yuy2Toyv12 (uint8_t *y, uint8_t *u, uint8_t *v, uint8_t *input,
        int width, int height)
{

  int    i, j, w2;

  w2 = width / 2;

  for (i = 0; i < height; i += 2) {
    for (j = 0; j < w2; j++) {
      /*
       * packed YUV 422 is: Y[i] U[i] Y[i+1] V[i]
       */
      *(y++) = *(input++);
      *(u++) = *(input++);
      *(y++) = *(input++);
      *(v++) = *(input++);
    }

    /*
     * down sampling
     */

    for (j = 0; j < w2; j++) {
      /*
       * skip every second line for U and V
       */
      *(y++) = *(input++);
      input++;
      *(y++) = *(input++);
      input++;
    }
  }
}

/*
 *   Create rgb data from yv12
 */

#define clip_8_bit(val)              \
{                                    \
   if (val < 0)                      \
      val = 0;                       \
   else                              \
      if (val > 255) val = 255;      \
}

uchar * yv12ToRgb (uint8_t *src_y, uint8_t *src_u, uint8_t *src_v,
           int width, int height)
{

  int     i, j;

  int     y, u, v;
  int     r, g, b;

  int     sub_i_uv;
  int     sub_j_uv;

  int     uv_width, uv_height;

  uchar *rgb;

  uv_width  = width / 2;
  uv_height = height / 2;

  rgb = new uchar[(width * height * 4)]; //qt needs a 32bit align
  if (!rgb)
  {
    kdError(555) << i18n("Not enough memory to make screenshot!") << "\n";
    return NULL;
  }  

  for (i = 0; i < height; ++i) {
    /*
     * calculate u & v rows
     */
    sub_i_uv = ((i * uv_height) / height);

    for (j = 0; j < width; ++j) {
      /*
       * calculate u & v columns
       */
      sub_j_uv = ((j * uv_width) / width);

      /***************************************************
       *
       *  Colour conversion from
       *    http://www.inforamp.net/~poynton/notes/colour_and_gamma/ColorFAQ.html#RTFToC30
       *
       *  Thanks to Billy Biggs <vektor@dumbterm.net>
       *  for the pointer and the following conversion.
       *
       *   R' = [ 1.1644         0    1.5960 ]   ([ Y' ]   [  16 ])
       *   G' = [ 1.1644   -0.3918   -0.8130 ] * ([ Cb ] - [ 128 ])
       *   B' = [ 1.1644    2.0172         0 ]   ([ Cr ]   [ 128 ])
       *
       *  Where in xine the above values are represented as
       *
       *   Y' == image->y
       *   Cb == image->u
       *   Cr == image->v
       *
       ***************************************************/

      y = src_y[(i * width) + j] - 16;
      u = src_u[(sub_i_uv * uv_width) + sub_j_uv] - 128;
      v = src_v[(sub_i_uv * uv_width) + sub_j_uv] - 128;

      r = (int)((1.1644 * (double)y) + (1.5960 * (double)v));
      g = (int)((1.1644 * (double)y) - (0.3918 * (double)u) - (0.8130 * (double)v));
      b = (int)((1.1644 * (double)y) + (2.0172 * (double)u));

      clip_8_bit (r);
      clip_8_bit (g);
      clip_8_bit (b);

      rgb[(i * width + j) * 4 + 0] = b; 
      rgb[(i * width + j) * 4 + 1] = g;
      rgb[(i * width + j) * 4 + 2] = r;
      rgb[(i * width + j) * 4 + 3] = 0;
      
    }
  }

  return rgb;
}


void VideoWindow::GetScreenshot(uchar*& rgb32BitData, int& videoWidth, int& videoHeight, double& scaleFactor) const
{

  uint8_t   *yuv = NULL, *y = NULL, *u = NULL, *v =NULL;
  
  int        width, height, ratio, format;
  double     desired_ratio, image_ratio;

  if (!xine_get_current_frame (xineStream, &width, &height, &ratio, &format, NULL))
    return;

  yuv = new uint8_t[((width+8) * (height+1) * 2)];
  if (yuv == NULL)
    {
      kdError(555) << i18n("Not enough memory to make screenshot!") << "\n";
      return;
    }  

  xine_get_current_frame (xineStream, &width, &height, &ratio, &format, yuv);

  videoWidth = width;
  videoHeight = height;

/*
 * convert to yv12 if necessary
 */

  switch (format) {
  case XINE_IMGFMT_YUY2:
    {
      uint8_t *yuy2 = yuv;

      yuv = new uint8_t[(width * height * 2)];
      if (yuv == NULL)
      {
        kdError(555) << i18n("Not enough memory to make screenshot!") << "\n";
        return;
      }  
      y = yuv;
      u = yuv + width * height;
      v = yuv + width * height * 5 / 4;

      yuy2Toyv12 (y, u, v, yuy2, width, height);

      delete [] yuy2;
    }
    break;
  case XINE_IMGFMT_YV12:
    y = yuv;
    u = yuv + width * height;
    v = yuv + width * height * 5 / 4;

    break;
  default:
    {
      kdWarning(555) << "Screenshot: Format " << (char *) &format << "not supported!\n";
      delete [] yuv;
      return;
    }  
  }

  /*
   * convert to rgb
   */

  rgb32BitData = yv12ToRgb (y, u, v, width, height);


  image_ratio = (double) width / (double) height;


  switch (ratio) {
  case XINE_VO_ASPECT_ANAMORPHIC:  /* anamorphic  */
  case XINE_VO_ASPECT_PAN_SCAN:    /* we display pan&scan as widescreen */
    desired_ratio = 16.0 /9.0;
    break;
  case XINE_VO_ASPECT_DVB:         /* 2.11:1 */
    desired_ratio = 2.11/1.0;
    break;
  case XINE_VO_ASPECT_SQUARE:      /* square pels */
  case XINE_VO_ASPECT_DONT_TOUCH:  /* probably non-mpeg stream => don't touch aspect ratio */
    desired_ratio = image_ratio;
    break;
  default:
    kdWarning(555) << "Unknown/vorbidden aspect ratio " << ratio << " in stream => using 4:3\n";
  case XINE_VO_ASPECT_4_3:         /* 4:3   */
    desired_ratio = 4.0 / 3.0;
    break;
  }

  scaleFactor = desired_ratio / image_ratio;

  delete [] yuv;
}

  

/**********/











